# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.util import CLIError

from azure_devtools.scenario_tests import AllowLargeResponse
from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer)


class NWConnectionMonitorScenarioTest(ScenarioTest):

    def _prepare_vm(self, resource_group, vm):
        vm_create_cmd_tpl = 'vm create -g {rg} --name {vm} ' \
                            '--image UbuntuLTS ' \
                            '--authentication-type password ' \
                            '--admin-username deploy ' \
                            '--admin-password PassPass10!) ' \
                            '--nsg {vm} ' \
                            '--nsg-rule None '
        vm_info = self.cmd(vm_create_cmd_tpl.format(rg=resource_group, vm=vm)).get_output_in_json()

        vm_enable_ext_tpl = 'vm extension set -g {rg} ' \
                            '--vm-name {vm} ' \
                            '--name NetworkWatcherAgentLinux ' \
                            '--publisher Microsoft.Azure.NetworkWatcher '
        self.cmd(vm_enable_ext_tpl.format(rg=resource_group, vm=vm))

        return vm_info

    def _prepare_connection_monitor_v2_env(self, resource_group, location):
        self.kwargs.update({
            'rg': resource_group,
            'location': location,
            'cmv2': 'CMv2-01',
            'test_group': 'DefaultTestGroup'
        })

        vm1_info = self._prepare_vm(resource_group, 'vm1')
        vm2_info = self._prepare_vm(resource_group, 'vm2')

        self.kwargs.update({
            'vm1_id': vm1_info['id'],
            'vm2_id': vm2_info['id'],
        })

        # create a connection monitor v2 with TCP monitoring
        self.cmd('network watcher connection-monitor create '
                 '--location {location} '
                 '--name {cmv2} '
                 '--endpoint-source-name vm1 '
                 '--endpoint-source-resource-id {vm1_id} '
                 '--endpoint-dest-name bing '
                 '--endpoint-dest-address bing.com '
                 '--test-config-name DefaultTestConfig '
                 '--protocol Tcp '
                 '--tcp-port 2048 ')

    @ResourceGroupPreparer(name_prefix='cli_test_nw_connection_monitor', location='westcentralus')
    @AllowLargeResponse()
    def test_nw_connection_monitor_v1(self, resource_group, resource_group_location):
        """
        This is copied from Azure/azure-cli,
        src/azure-cli/azure/cli/command_modules/network/tests/latest/test_network_commands.py,
        test_network_watcher_connection_monitor()
        """
        import time
        self.kwargs.update({
            'loc': resource_group_location,
            'vm2': 'vm2',
            'vm3': 'vm3',
            'cm': 'cm1'
        })

        self.cmd('vm create -g {rg} -n {vm2} '
                 '--image UbuntuLTS '
                 '--authentication-type password '
                 '--admin-username deploy '
                 '--admin-password PassPass10!) '
                 '--nsg {vm2} '
                 '--nsg-rule None')
        self.cmd('vm extension set -g {rg} '
                 '--vm-name {vm2} '
                 '-n NetworkWatcherAgentLinux '
                 '--publisher Microsoft.Azure.NetworkWatcher')
        self.cmd('vm create -g {rg} -n {vm3} '
                 '--image UbuntuLTS '
                 '--authentication-type password '
                 '--admin-username deploy '
                 '--admin-password PassPass10!) '
                 '--nsg {vm3} '
                 '--nsg-rule None')
        self.cmd('vm extension set -g {rg} '
                 '--vm-name {vm3} '
                 '-n NetworkWatcherAgentLinux '
                 '--publisher Microsoft.Azure.NetworkWatcher')
        time.sleep(20)
        self.cmd('network watcher connection-monitor create '
                 '-n {cm} '
                 '--source-resource {vm2} '
                 '-g {rg} '
                 '--dest-resource {vm3} '
                 '--dest-port 80 '
                 '--tags foo=doo')
        self.cmd('network watcher connection-monitor list -l {loc}')
        self.cmd('network watcher connection-monitor show -l {loc} -n {cm}')
        try:
            self.cmd('network watcher connection-monitor stop -l {loc} -n {cm}')
            self.cmd('network watcher connection-monitor start -l {loc} -n {cm}')
        except CLIError:
            pass
        self.cmd('network watcher connection-monitor query -l {loc} -n {cm}')
        self.cmd('network watcher connection-monitor delete -l {loc} -n {cm}')

    @ResourceGroupPreparer(name_prefix='connection_monitor_v2_test_', location='eastus')
    @AllowLargeResponse()
    def test_nw_connection_monitor_v2_creation(self, resource_group, resource_group_location):
        # create a V2 connection monitor with --location and TCP configuration
        self._prepare_connection_monitor_v2_env(resource_group, resource_group_location)

        # create a v2 connection monitor without --location and ICMP configuration
        self.cmd('network watcher connection-monitor create '
                 '--name cmv2-01 '
                 '--endpoint-source-name vm2 '
                 '--endpoint-source-resource-id {vm2_id} '
                 '--endpoint-dest-name bing '
                 '--endpoint-dest-address bing.com '
                 '--test-config-name DefaultIcmp '
                 '--protocol Icmp ')

        self.cmd('network watcher connection-monitor list -l {location}')
        self.cmd('network watcher connection-monitor show -l {location} -n {cmv2}')
        self.cmd('network watcher connection-monitor show -l {location} -n cmv2-01')

    @ResourceGroupPreparer(name_prefix='connection_monitor_v2_test_', location='eastus')
    @AllowLargeResponse()
    def test_nw_connection_monitor_v2_endpoint(self, resource_group, resource_group_location):
        self._prepare_connection_monitor_v2_env(resource_group, resource_group_location)

        self.cmd('network vnet create -g {rg} --name cmv2-vnet --subnet-name subnet1 --address-prefix 10.0.0.0/24')
        subnet = self.cmd('network vnet subnet show -g {rg} --vnet-name cmv2-vnet --name subnet1').get_output_in_json()
        self.kwargs.update({
            'cm_subnet_endpoint': 'cm_subnet_endpoint',
            'subnet_id': subnet['id']
        })

        # add an endpoint as source
        self.cmd('network watcher connection-monitor endpoint add '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--source-test-groups {test_group} '
                 '--name {cm_subnet_endpoint} '
                 '--resource-id {subnet_id} '
                 '--type AzureSubnet '
                 '--address-exclude 10.0.0.25 '
                 '--coverage-level BelowAverage')

        # add an endpoint as destination
        self.cmd('network watcher connection-monitor endpoint add '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--dest-test-groups {test_group} '
                 '--name Github '
                 '--address github.com '
                 '--type ExternalAddress')

        self.cmd('network watcher connection-monitor endpoint list --connection-monitor {cmv2} --location {location}',
                 checks=self.check('length(@)', 4))
        self.cmd('network watcher connection-monitor endpoint show '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--name Github')

        # remove one
        self.cmd('network watcher connection-monitor endpoint remove '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--name {cm_subnet_endpoint} '
                 '--test-groups DefaultTestGroup ')
        self.cmd('network watcher connection-monitor endpoint list --connection-monitor {cmv2} --location {location}',
                 checks=self.check('length(@)', 3))

    @ResourceGroupPreparer(name_prefix='connection_monitor_v2_test_', location='eastus')
    @AllowLargeResponse()
    def test_nw_connection_monitor_v2_test_configuration(self, resource_group, resource_group_location):
        self._prepare_connection_monitor_v2_env(resource_group, resource_group_location)

        # add a TCP test configuration
        self.cmd('network watcher connection-monitor test-configuration add '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--name TCPConfig '
                 '--protocol Tcp '
                 '--test-groups DefaultTestGroup '
                 '--tcp-port 8080 '
                 '--tcp-port-behavior ListenIfAvailable '
                 '--tcp-disable-trace-route true '
                 '--frequency 120 '
                 '--threshold-round-trip-time 1200')

        # add a HTTP test configuration
        self.cmd('network watcher connection-monitor test-configuration add '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--name HTTPConfig '
                 '--test-groups DefaultTestGroup '
                 '--protocol Http '
                 '--frequency 90 '
                 '--http-method Get '
                 '--http-path "/" '
                 '--http-valid-status-codes 200 301 '
                 '--http-request-header name=Host value=azure.com '
                 '--http-request-header name=UserAgent value=AzureCLITest ')
        self.cmd('network watcher connection-monitor test-configuration list '
                 '--connection-monitor {cmv2} '
                 '--location {location} ',
                 checks=self.check('length(@)', 3))
        self.cmd('network watcher connection-monitor test-configuration show '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--name HTTPConfig ')

        # add a ICMP test configuration
        self.cmd('network watcher connection-monitor test-configuration add '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--name ICMPConfig '
                 '--test-groups DefaultTestGroup '
                 '--protocol Icmp ')
        self.cmd('network watcher connection-monitor test-configuration list '
                 '--connection-monitor {cmv2} '
                 '--location {location} ',
                 checks=self.check('length(@)', 4))
        self.cmd('network watcher connection-monitor test-configuration show '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--name ICMPConfig ')

        self.cmd('network watcher connection-monitor test-configuration remove '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--name HTTPConfig ')
        self.cmd('network watcher connection-monitor test-configuration list '
                 '--connection-monitor {cmv2} '
                 '--location {location} ',
                 checks=self.check('length(@)', 3))

    @ResourceGroupPreparer(name_prefix='connection_monitor_v2_test_', location='eastus')
    @AllowLargeResponse()
    def test_connection_monitor_v2_test_group(self, resource_group, resource_group_location):
        self._prepare_connection_monitor_v2_env(resource_group, resource_group_location)

        # add an endpoint as source for later use
        self.cmd('network watcher connection-monitor endpoint add '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--source-test-groups {test_group} '
                 '--name vm02 '
                 '--resource-id {vm2_id} '
                 '--filter-type Include '
                 '--filter-item type=AgentAddress address=10.0.0.1 '
                 '--filter-item type=AgentAddress address=10.0.0.2 ')

        # add an endpoint as destination for later use
        self.cmd('network watcher connection-monitor endpoint add '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--dest-test-groups {test_group} '
                 '--name Github '
                 '--address github.com ')

        # add a HTTP test configuration for later use
        self.cmd('network watcher connection-monitor test-configuration add '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--name HTTPConfig '
                 '--test-groups DefaultTestGroup '
                 '--protocol Http '
                 '--frequency 90 '
                 '--http-method Get '
                 '--http-path "/" '
                 '--http-valid-status-codes 200 301 '
                 '--http-request-header name=Host value=azure.com '
                 '--http-request-header name=UserAgent value=AzureCLITest ')

        # add a ICMP test configuration
        self.cmd('network watcher connection-monitor test-configuration add '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--name ICMPConfig '
                 '--test-groups DefaultTestGroup '
                 '--protocol Icmp ')
        self.cmd('network watcher connection-monitor test-configuration list '
                 '--connection-monitor {cmv2} '
                 '--location {location} ',
                 checks=self.check('length(@)', 3))
        self.cmd('network watcher connection-monitor test-configuration show '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--name ICMPConfig ')

        # add a test group with existing source endpoint, new-added destination endpoints and existing test config
        self.cmd('network watcher connection-monitor test-group add '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--name RouteTestGroup '
                 '--disable false '
                 '--endpoint-source-name vm1 '
                 '--endpoint-dest-name aks.ms '
                 '--endpoint-dest-address aks.ms '
                 '--test-config-name ICMPConfig ')
        self.cmd('network watcher connection-monitor test-group list '
                 '--connection-monitor {cmv2} '
                 '--location {location} ',
                 checks=self.check('length(@)', 2))
        self.cmd('network watcher connection-monitor test-group show '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--name RouteTestGroup ')

        self.cmd('network watcher connection-monitor test-group remove '
                 '--connection-monitor {cmv2} '
                 '--location {location} '
                 '--name DefaultTestGroup ')
        self.cmd('network watcher connection-monitor test-group list '
                 '--connection-monitor {cmv2} '
                 '--location {location} ',
                 checks=self.check('length(@)', 1))

    @ResourceGroupPreparer(name_prefix='connection_monitor_v2_test_', location='eastus')
    @AllowLargeResponse()
    def test_nw_connection_monitor_output(self, resource_group, resource_group_location):
        self._prepare_connection_monitor_v2_env(resource_group, resource_group_location)

        self.kwargs.update({
            'workspace_name': self.create_random_name('clitest', 20)
        })

        workspace = self.cmd('monitor log-analytics workspace create '
                             '-g {rg} '
                             '--location {location} '
                             '--workspace-name {workspace_name} ').get_output_in_json()

        self.kwargs.update({
            'workspace_id': workspace['id']
        })

        self.cmd('network watcher connection-monitor output list '
                 '--location {location} '
                 '--connection-monitor {cmv2} ')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, JMESPathCheck


class TestActionGroupScenarios(ScenarioTest):
    @ResourceGroupPreparer(location='southcentralus')
    def test_monitor_action_group_basic_scenario(self, resource_group):
        # the prefix is intentionally keep long so as to test the default short name conversion
        action_group_name = self.create_random_name('cliactiongrouptest', 32)
        self.cmd('monitor action-group create -n {} -g {} -ojson'.format(action_group_name, resource_group), checks=[
            JMESPathCheck('length(emailReceivers)', 0),
            JMESPathCheck('length(smsReceivers)', 0),
            JMESPathCheck('length(webhookReceivers)', 0),
            JMESPathCheck('location', 'Global'),
            JMESPathCheck('name', action_group_name),
            JMESPathCheck('groupShortName', action_group_name[:12]),
            JMESPathCheck('enabled', True),
            JMESPathCheck('resourceGroup', resource_group),
            JMESPathCheck('tags', None)
        ])

        self.cmd('monitor action-group list -g {} -ojson'.format(resource_group),
                 checks=[JMESPathCheck('type(@)', 'array'),
                         JMESPathCheck('length(@)', 1),
                         JMESPathCheck('[0].name', action_group_name)])

        self.cmd('monitor action-group update -n {} -g {} -ojson --short-name new_name --tag owner=alice'
                 .format(action_group_name, resource_group), checks=[JMESPathCheck('tags.owner', 'alice'),
                                                                     JMESPathCheck('groupShortName', 'new_name')])

        self.cmd('monitor action-group update -n {} -g {} -ojson -a email alice alice@example.com usecommonalertsChema '
                 '-a sms alice_sms 1 5551234567 '
                 '-a webhook alice_web https://www.example.com/alert?name=alice useAadAuth testobjid http://iduri usecommonalertschema '
                 '-a armrole alicearmrole abcde usecommonAlertSchema '
                 '-a azureapppush alice_apppush alice@example.com '
                 '-a itsm alice_itsm test_workspace test_conn ticket_blob eastus '
                 '-a automationrunbook alice_runbook test_account test_runbook test_webhook http://example.com isglobalrunbook '
                 '-a voice alice_voice 1 5551234567 '
                 '-a logicapp alice_logicapp test_resource http://callback '
                 '-a azurefunction azfunc test_rsrc test_func http://trigger usecommonalertSchema '
                 .format(action_group_name, resource_group), checks=[JMESPathCheck('length(emailReceivers)', 1),
                                                                     JMESPathCheck('length(smsReceivers)', 1),
                                                                     JMESPathCheck('length(webhookReceivers)', 1),
                                                                     JMESPathCheck('length(armRoleReceivers)', 1),
                                                                     JMESPathCheck('length(azureAppPushReceivers)', 1),
                                                                     JMESPathCheck('length(itsmReceivers)', 1),
                                                                     JMESPathCheck('length(automationRunbookReceivers)', 1),
                                                                     JMESPathCheck('length(voiceReceivers)', 1),
                                                                     JMESPathCheck('length(logicAppReceivers)', 1),
                                                                     JMESPathCheck('length(azureFunctionReceivers)', 1)])

        self.cmd('monitor action-group update -n {} -g {} -ojson -r alice_web'
                 .format(action_group_name, resource_group), checks=[JMESPathCheck('length(emailReceivers)', 1),
                                                                     JMESPathCheck('length(smsReceivers)', 1),
                                                                     JMESPathCheck('length(webhookReceivers)', 0)])

        self.cmd('monitor action-group enable-receiver -n nonexist --action-group {} -g {}'
                 .format(action_group_name, resource_group), expect_failure=True)

        self.cmd('monitor action-group enable-receiver -n alice --action-group {} -g {}'
                 .format(action_group_name, resource_group))

        self.cmd('monitor action-group delete -n {} -g {} -ojson'.format(action_group_name, resource_group))

        self.cmd('monitor action-group list -g {} -ojson'.format(resource_group),
                 checks=[JMESPathCheck('type(@)', 'array'),
                         JMESPathCheck('length(@)', 0)])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

"""
Support styled output.

Currently, only color is supported, underline/bold/italic may be supported in the future.

Design spec:
https://devdivdesignguide.azurewebsites.net/command-line-interface/color-guidelines-for-command-line-interface/

For a complete demo, see `src/azure-cli/azure/cli/command_modules/util/custom.py` and run `az demo style`.
"""

import sys
from enum import Enum

from colorama import Fore


class Style(str, Enum):
    PRIMARY = "primary"
    SECONDARY = "secondary"
    IMPORTANT = "important"
    ACTION = "action"  # name TBD
    HYPERLINK = "hyperlink"
    # Message colors
    ERROR = "error"
    SUCCESS = "success"
    WARNING = "warning"


THEME = {
    # Style to ANSI escape sequence mapping
    # https://docs.microsoft.com/en-us/windows/console/console-virtual-terminal-sequences
    Style.PRIMARY: Fore.LIGHTWHITE_EX,
    Style.SECONDARY: Fore.LIGHTBLACK_EX,  # may use WHITE, but will lose contrast to LIGHTWHITE_EX
    Style.IMPORTANT: Fore.LIGHTMAGENTA_EX,
    Style.ACTION: Fore.LIGHTBLUE_EX,
    Style.HYPERLINK: Fore.LIGHTCYAN_EX,
    # Message colors
    Style.ERROR: Fore.LIGHTRED_EX,
    Style.SUCCESS: Fore.LIGHTGREEN_EX,
    Style.WARNING: Fore.LIGHTYELLOW_EX,
}


def print_styled_text(styled, file=sys.stderr):
    formatted = format_styled_text(styled)
    print(formatted, file=file)


def format_styled_text(styled_text):
    # https://python-prompt-toolkit.readthedocs.io/en/stable/pages/printing_text.html#style-text-tuples
    formatted_parts = []

    for text in styled_text:
        # str can also be indexed, bypassing IndexError, so explicitly check if the type is tuple
        if not (isinstance(text, tuple) and len(text) == 2):
            from azure.cli.core.azclierror import CLIInternalError
            raise CLIInternalError("Invalid styled text. It should be a list of 2-element tuples.")

        style = text[0]
        if style not in THEME:
            from azure.cli.core.azclierror import CLIInternalError
            raise CLIInternalError("Invalid style. Only use pre-defined style in Style enum.")

        formatted_parts.append(THEME[text[0]] + text[1])

    # Reset control sequence
    formatted_parts.append(Fore.RESET)
    return ''.join(formatted_parts)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer
from azure_devtools.scenario_tests import AllowLargeResponse


class SecurityCenterIotTests(ScenarioTest):

    @AllowLargeResponse()
    @ResourceGroupPreparer(location='eastus')
    def test_security_iot(self):
        self.kwargs.update({
            'iot_hub_name': self.create_random_name(prefix='azurecli-hub', length=24)
        })
        iot_hub_id = self.cmd('az iot hub create -n {iot_hub_name} -g {rg}').get_output_in_json()['id']
        self.kwargs.update({
            'iot_hub_id': iot_hub_id
        })

        self.cmd('az security iot-solution create --solution-name "azurecli-hub" --resource-group {rg} --iot-hubs {iot_hub_id} --display-name "Solution Default" --location "east us"')
        azure_cli_new_iot_security_solution = self.cmd('az security iot-solution list --resource-group {rg}').get_output_in_json()
        assert len(azure_cli_new_iot_security_solution) == 1

        self.cmd('az security iot-solution update --solution-name "azurecli-hub" --resource-group {rg} --iot-hubs {iot_hub_id} --display-name "Solution Default"')
        azure_cli_new_iot_security_solution = self.cmd('az security iot-solution list --resource-group {rg}').get_output_in_json()
        assert len(azure_cli_new_iot_security_solution) == 1

        azure_cli_iot_security_solution = self.cmd('az security iot-solution show --solution-name "azurecli-hub" --resource-group {rg}').get_output_in_json()
        assert len(azure_cli_iot_security_solution) >= 1

        self.cmd('az security iot-solution delete --solution-name "azurecli-hub" --resource-group {rg}')

        azure_cli_no_iot_security_solution = self.cmd('az security iot-solution list --resource-group {rg}').get_output_in_json()
        assert len(azure_cli_no_iot_security_solution) == 0



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=import-error, protected-access
# Justification:
#   * The azure.cli packages are not visible while the script is scaned but they are guaranteed
#     existence on the droid image
#   * The test method names and other properties are protected for unit test framework.

import os.path
from pkgutil import iter_modules
from unittest import TestLoader
from importlib import import_module
from json import dumps as json_dumps

import azure.cli
from azure.cli.testsdk import ScenarioTest, LiveScenarioTest

RECORDS = []


def get_test_type(test_case):
    if isinstance(test_case, ScenarioTest):
        return 'Recording'
    elif isinstance(test_case, LiveScenarioTest):
        return 'Live'
    return 'Unit'


def find_recording_file(test_path):
    module_path, _, test_method = test_path.rsplit('.', 2)
    test_folder = os.path.dirname(import_module(module_path).__file__)
    recording_file = os.path.join(test_folder, 'recordings', test_method + '.yaml')
    return recording_file if os.path.exists(recording_file) else None


def search(path, prefix=''):
    loader = TestLoader()
    for _, name, is_pkg in iter_modules(path):
        full_name = '{}.{}'.format(prefix, name)
        module_path = os.path.join(path[0], name)

        if is_pkg:
            search([module_path], full_name)

        if not is_pkg and name.startswith('test'):
            test_module = import_module(full_name)
            for suite in loader.loadTestsFromModule(test_module):
                for test in suite._tests:
                    path = '{}.{}.{}'.format(full_name, test.__class__.__name__, test._testMethodName)
                    rec = {
                        'ver': '1.0',
                        'execution': {
                            'command': 'python -m unittest {}'.format(path),
                            'recording': find_recording_file(path)
                        },
                        'classifier': {
                            'identifier': path,
                            'type': get_test_type(test),
                        }
                    }
                    RECORDS.append(rec)


search(azure.cli.__path__, 'azure.cli')

print(json_dumps(RECORDS))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer,
                               JMESPathCheck, NoneCheck, StringCheck, StringContainCheck)
from ..storage_test_util import StorageScenarioMixin


class StorageFileShareScenarios(StorageScenarioMixin, ScenarioTest):
    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_storage_file_copy_scenario(self, resource_group, storage_account):
        account_info = self.get_account_info(resource_group, storage_account)
        s1 = self.create_share(account_info)
        s2 = self.create_share(account_info)
        d1 = 'dir1'
        d2 = 'dir2'

        self.storage_cmd('storage directory create --share-name {} -n {}', account_info, s1, d1)
        self.storage_cmd('storage directory create --share-name {} -n {}', account_info, s2, d2)

        local_file = self.create_temp_file(512, full_random=False)
        src_file = os.path.join(d1, 'source_file.txt')
        dst_file = os.path.join(d2, 'destination_file.txt')

        self.storage_cmd('storage file upload -p "{}" --share-name {} --source "{}"', account_info,
                         src_file, s1, local_file)
        self.storage_cmd('storage file exists -p "{}" -s {}', account_info, src_file, s1) \
            .assert_with_checks(JMESPathCheck('exists', True))

        # test that a file can be successfully copied to root
        src_uri = self.storage_cmd('storage file url -p "{}" -s {}', account_info, src_file,
                                   s1).output
        self.assertTrue(src_uri)

        copy_id = self.storage_cmd('storage file copy start -s {} -p "{}" -u {}',
                                   account_info, s2, dst_file, src_uri) \
            .assert_with_checks(JMESPathCheck('status', 'success')) \
            .get_output_in_json()['id']

        self.storage_cmd('storage file show --share-name {} -p "{}"', account_info, s2, dst_file) \
            .assert_with_checks(JMESPathCheck('name', os.path.basename(dst_file)),
                                JMESPathCheck('properties.copy.id', copy_id),
                                JMESPathCheck('properties.copy.status', 'success'))

        # test that a file can be successfully copied to a directory
        copy_id = \
            self.storage_cmd('storage file copy start -s {} -p "{}" -u {}', account_info, s2,
                             dst_file,
                             src_uri).assert_with_checks(
                JMESPathCheck('status', 'success')).get_output_in_json()['id']

        self.storage_cmd('storage file show -s {} -p "{}"', account_info, s2, dst_file) \
            .assert_with_checks(JMESPathCheck('name', os.path.basename(dst_file)),
                                JMESPathCheck('properties.copy.id', copy_id),
                                JMESPathCheck('properties.copy.status', 'success'))

        # test that a file can be successfully copied by name components
        self.storage_cmd('storage file copy start -s {} -p "{}" --source-share {} --source-path "{}"',
                         account_info, s2, dst_file, s1, src_file) \
            .assert_with_checks(JMESPathCheck('status', 'success'))

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_storage_file_main_scenario(self, resource_group, storage_account):
        account_info = self.get_account_info(resource_group, storage_account)
        s1 = self.create_share(account_info)
        s2 = self.create_random_name('share', 24)
        self.storage_cmd('storage share create -n {} --metadata foo=bar cat=hat', account_info, s2)
        self.storage_cmd('storage share exists -n {}', account_info, s1) \
            .assert_with_checks(JMESPathCheck('exists', True))
        self.storage_cmd('storage share metadata show --name {}', account_info, s2) \
            .assert_with_checks(JMESPathCheck('cat', 'hat'), JMESPathCheck('foo', 'bar'))

        share_list = self.storage_cmd('storage share list --query "[].name"',
                                      account_info).get_output_in_json()
        self.assertIn(s1, share_list)
        self.assertIn(s2, share_list)

        # verify metadata can be set, queried, and cleared
        self.storage_cmd('storage share metadata update --name {} --metadata a=b c=d', account_info, s1)
        self.storage_cmd('storage share metadata show -n {}', account_info, s1) \
            .assert_with_checks(JMESPathCheck('a', 'b'), JMESPathCheck('c', 'd'))
        self.storage_cmd('storage share metadata update -n {}', account_info, s1)
        self.storage_cmd('storage share metadata show -n {}', account_info, s1) \
            .assert_with_checks(NoneCheck())

        self.storage_cmd('storage share update --name {} --quota 3', account_info, s1)
        self.storage_cmd('storage share show --name {}', account_info, s1) \
            .assert_with_checks(JMESPathCheck('properties.quota', 3))
        self.storage_cmd('storage share url --name {}', account_info, s1) \
            .assert_with_checks(StringContainCheck(s1), StringContainCheck('http'))
        unc = self.storage_cmd('storage share url --name {} --unc', account_info, s1).output
        self.assertTrue('http' not in unc and s1 in unc)

        sas = self.storage_cmd('storage share generate-sas -n {} --permissions r --expiry '
                               '2046-08-23T10:30Z', account_info, s1).output
        self.assertIn('sig=', sas)

        self.validate_file_scenario(account_info, s1)
        self.validate_directory_scenario(account_info, s1)

        self.storage_cmd('storage share delete -n {}', account_info, s1) \
            .assert_with_checks(JMESPathCheck('deleted', True))

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_storage_file_copy_snapshot_scenario(self, resource_group, storage_account):
        account_info = self.get_account_info(resource_group, storage_account)
        s1 = self.create_share(account_info)
        s2 = self.create_share(account_info)
        d1 = 'dir1'
        d2 = 'dir2'

        self.storage_cmd('storage directory create --share-name {} -n {}', account_info, s1, d1)
        self.storage_cmd('storage directory create --share-name {} -n {}', account_info, s2, d2)

        local_file = self.create_temp_file(512, full_random=False)
        src_file = os.path.join(d1, 'source_file.txt')
        dst_file = os.path.join(d2, 'destination_file.txt')

        self.storage_cmd('storage file upload -p "{}" --share-name {} --source "{}"', account_info,
                         src_file, s1, local_file)
        self.storage_cmd('storage file exists -p "{}" -s {}', account_info, src_file, s1) \
            .assert_with_checks(JMESPathCheck('exists', True))

        snapshot = self.storage_cmd('storage share snapshot -n {}', account_info,
                                    s1).get_output_in_json()['snapshot']

        # remove the source file from share
        self.storage_cmd('storage file delete --share-name {} -p "{}"',
                         account_info, s1, src_file)
        self.storage_cmd('storage file exists -p "{}" -s {}', account_info, src_file, s1) \
            .assert_with_checks(JMESPathCheck('exists', False))

        copy_id = self.storage_cmd('storage file copy start -s {} -p "{}" --source-share {} --source-path "{}" --file-snapshot {}',
                                   account_info, s2, dst_file, s1, src_file, snapshot) \
            .assert_with_checks(JMESPathCheck('status', 'success')) \
            .get_output_in_json()['id']

        self.storage_cmd('storage file show --share-name {} -p "{}"', account_info, s2, dst_file) \
            .assert_with_checks(JMESPathCheck('name', os.path.basename(dst_file)),
                                JMESPathCheck('properties.copy.id', copy_id),
                                JMESPathCheck('properties.copy.status', 'success'))

    def validate_directory_scenario(self, account_info, share):
        directory = self.create_random_name('dir', 16)
        self.storage_cmd('storage directory create --share-name {} --name {} --fail-on-exist',
                         account_info, share, directory) \
            .assert_with_checks(JMESPathCheck('created', True))
        self.storage_cmd('storage directory list -s {}', account_info, share) \
            .assert_with_checks(JMESPathCheck('length(@)', 1))
        self.storage_cmd('storage directory exists --share-name {} -n {}',
                         account_info, share, directory) \
            .assert_with_checks(JMESPathCheck('exists', True))
        self.storage_cmd('storage directory metadata update -s {} -n {} --metadata a=b c=d',
                         account_info, share, directory)
        self.storage_cmd('storage directory metadata show --share-name {} -n {}',
                         account_info, share, directory) \
            .assert_with_checks(JMESPathCheck('a', 'b'), JMESPathCheck('c', 'd'))
        self.storage_cmd('storage directory show --share-name {} -n {}',
                         account_info, share, directory) \
            .assert_with_checks(JMESPathCheck('metadata', {'a': 'b', 'c': 'd'}),
                                JMESPathCheck('name', directory))
        self.storage_cmd('storage directory metadata update --share-name {} --name {}',
                         account_info, share, directory)
        self.storage_cmd('storage directory metadata show --share-name {} --name {}',
                         account_info, share, directory).assert_with_checks(NoneCheck())

        self.validate_subdir_scenario(account_info, share, directory)

        self.storage_cmd('storage directory delete --share-name {} --name {} --fail-not-exist',
                         account_info, share, directory) \
            .assert_with_checks(JMESPathCheck('deleted', True))
        self.storage_cmd('storage directory exists --share-name {} --name {}',
                         account_info, share, directory) \
            .assert_with_checks(JMESPathCheck('exists', False))

        # verify a directory can be created with metadata and then delete
        directory = self.create_random_name('dir', 16)
        self.storage_cmd('storage directory create --share-name {} --name {} --fail-on-exist '
                         '--metadata foo=bar cat=hat', account_info, share, directory) \
            .assert_with_checks(JMESPathCheck('created', True))
        self.storage_cmd('storage directory metadata show --share-name {} -n {}',
                         account_info, share, directory) \
            .assert_with_checks(JMESPathCheck('cat', 'hat'), JMESPathCheck('foo', 'bar'))
        self.storage_cmd('storage directory delete --share-name {} --name {} --fail-not-exist',
                         account_info, share, directory) \
            .assert_with_checks(JMESPathCheck('deleted', True))

    def validate_file_scenario(self, account_info, share):
        source_file = self.create_temp_file(128, full_random=False)
        dest_file = self.create_temp_file(1)
        filename = "sample_file.bin"

        self.storage_cmd('storage file upload --share-name {} --source "{}" -p {}', account_info,
                         share, source_file, filename)
        self.storage_cmd('storage file exists -s {} -p {}', account_info, share, filename) \
            .assert_with_checks(JMESPathCheck('exists', True))

        if os.path.isfile(dest_file):
            os.remove(dest_file)

        self.storage_cmd('storage file download --share-name {} -p "{}" --dest "{}"', account_info,
                         share, filename, dest_file)

        self.assertTrue(os.path.isfile(dest_file))
        self.assertEqual(os.stat(dest_file).st_size, 128 * 1024)

        self.storage_cmd('storage file download --share-name {} -p "{}" --dest "{}" --start-range 0 --end-range 511', account_info,
                         share, filename, dest_file)

        self.assertTrue(os.path.isfile(dest_file))
        self.assertEqual(os.stat(dest_file).st_size, 512)

        # test resize command
        self.storage_cmd('storage file resize --share-name {} -p "{}" --size 1234', account_info,
                         share, filename)
        self.storage_cmd('storage file show -s {} -p "{}"', account_info, share, filename) \
            .assert_with_checks(JMESPathCheck('properties.contentLength', 1234))

        # test ability to set and reset metadata
        self.storage_cmd('storage file metadata update --share-name {} -p "{}" --metadata a=b c=d',
                         account_info, share, filename)
        self.storage_cmd('storage file metadata show -s {} -p "{}"', account_info, share,
                         filename) \
            .assert_with_checks(JMESPathCheck('a', 'b'), JMESPathCheck('c', 'd'))
        self.storage_cmd('storage file metadata update --share-name {} -p "{}"', account_info,
                         share, filename)
        self.storage_cmd('storage file metadata show -s {} -p "{}"', account_info, share,
                         filename) \
            .assert_with_checks(NoneCheck())

        file_url = 'https://{}.file.core.windows.net/{}/{}'.format(account_info[0], share, filename)
        self.storage_cmd('storage file url -s {} -p "{}"', account_info, share, filename) \
            .assert_with_checks(StringCheck(file_url))

        self.assertIn(filename,
                      self.storage_cmd('storage file list -s {} --query "[].name"',
                                       account_info, share).get_output_in_json())

        from datetime import datetime, timedelta
        expiry = (datetime.utcnow() + timedelta(hours=1)).strftime('%Y-%m-%dT%H:%MZ')
        self.storage_cmd('storage file generate-sas -s {} -p {} --permissions r --expiry {}', account_info, share,
                         filename, expiry).assert_with_checks(StringContainCheck('sig='))

        self.storage_cmd('storage file update -s {} -p {} --content-type "test/type"', account_info,
                         share, filename)
        self.storage_cmd('storage file show -s {} -p {}', account_info, share,
                         filename) \
            .assert_with_checks(JMESPathCheck('properties.contentSettings.contentType',
                                              'test/type'))

        self.storage_cmd('storage file delete --share-name {} -p "{}"',
                         account_info, share, filename)
        self.storage_cmd('storage file exists --share-name {} -p "{}"',
                         account_info, share, filename) \
            .assert_with_checks(JMESPathCheck('exists', False))

    def validate_subdir_scenario(self, account_info, share, directory):
        source_file = self.create_temp_file(64, full_random=False)
        dest_file = self.create_temp_file(1)
        filename = 'testfile.txt'

        self.storage_cmd('storage file upload --share-name {} -p "{}/{}" --source "{}"',
                         account_info, share, directory, filename, source_file)
        self.storage_cmd('storage file exists --share-name {} -p "{}/{}"',
                         account_info, share, directory, filename) \
            .assert_with_checks(JMESPathCheck('exists', True))

        self.storage_cmd('storage file download --share-name {} -p "{}/{}" --dest "{}"',
                         account_info, share, directory, filename, dest_file)

        self.assertTrue(os.path.isfile(dest_file))
        self.assertEqual(os.stat(dest_file).st_size, 64 * 1024)

        self.assertIn(filename, self.storage_cmd('storage file list -s {} -p {} --query "[].name"',
                                                 account_info, share, directory)
                      .get_output_in_json())

        self.storage_cmd('storage share stats --name {}', account_info, share) \
            .assert_with_checks(StringCheck('1'))
        self.storage_cmd('storage file delete --share-name {} -p "{}/{}"',
                         account_info, share, directory, filename)
        self.storage_cmd('storage file exists -s {} -p "{}"', account_info, share, filename) \
            .assert_with_checks(JMESPathCheck('exists', False))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# AZURE CLI EventHub - NAMESPACE TEST DEFINITIONS

import time

from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer)


# pylint: disable=line-too-long
# pylint: disable=too-many-lines


class EHNetworkCURDScenarioTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_eh_network')
    def test_eh_network(self, resource_group):
        self.kwargs.update({
            'namespacename': self.create_random_name(prefix='eventhubs-cli', length=20),
            'namespacenamekafka': self.create_random_name(prefix='eventhubs-cli1', length=20),
            'tags': {'tag1=value1'},
            'tags2': {'tag2=value2'},
            'sku': 'Standard',
            'tier': 'Standard',
            'authoname': self.create_random_name(prefix='cliAutho', length=20),
            'defaultauthorizationrule': 'RootManageSharedAccessKey',
            'accessrights': 'Send',
            'namevnet': 'sbehvnettest1',
            'namevnet1': 'sbehvnettest2',
            'namesubnet1': 'default',
            'namesubnet2': 'secondvnet',
            'isautoinflateenabled': 'True',
            'maximumthroughputunits': 4,
            'maximumthroughputunits_update': 5,
            'varfalse': 'false',
            'ipmask1': '1.1.1.1',
            'ipmask2': '2.2.2.2'
        })

        self.cmd('network vnet create --resource-group {rg} --name {namevnet}')
        self.cmd('network vnet create --resource-group {rg} --name {namevnet1}')

        created_subnet1 = self.cmd(
            'network vnet subnet create --resource-group {rg} --name {namesubnet1} --vnet-name {namevnet} --address-prefixes 10.0.0.0/24').get_output_in_json()
        created_subnet2 = self.cmd(
            'network vnet subnet create --resource-group {rg} --name {namesubnet2} --vnet-name {namevnet1} --address-prefixes 10.0.0.0/24').get_output_in_json()

        # Check for the NameSpace name Availability
        self.cmd('eventhubs namespace exists --name {namespacename}',
                 checks=[self.check('nameAvailable', True)])

        # Create Namespace
        self.cmd(
            'eventhubs namespace create --resource-group {rg} --name {namespacename} --tags {tags} --sku {sku} --default-action Allow',
            checks=[self.check('sku.name', '{sku}')])

        # Get Created Namespace
        self.cmd('eventhubs namespace show --resource-group {rg} --name {namespacename}',
                 checks=[self.check('sku.name', '{sku}')])

        # Update Namespace
        self.cmd(
            'eventhubs namespace update --resource-group {rg} --name {namespacename} --tags {tags} --default-action Deny',
            checks=[self.check('sku.name', '{sku}')])

        # Get NetworkRule
        self.cmd(
            'eventhubs namespace network-rule list --resource-group {rg} --name {namespacename}').get_output_in_json()

        # add IP Rule
        iprule = self.cmd(
            'eventhubs namespace network-rule add --resource-group {rg} --name {namespacename} --ip-address {ipmask1} --action Allow').get_output_in_json()
        self.assertEqual(len(iprule['ipRules']), 1)

        # add IP Rule
        iprule = self.cmd(
            'eventhubs namespace network-rule add --resource-group {rg} --name {namespacename} --ip-address {ipmask2} --action Allow').get_output_in_json()
        self.assertEqual(len(iprule['ipRules']), 2)
        self.assertTrue(iprule['ipRules'][0]['ipMask'] == '1.1.1.1')
        self.assertTrue(iprule['ipRules'][1]['ipMask'] == '2.2.2.2')

        # Get list of IP rule
        iprule = self.cmd(
            'eventhubs namespace network-rule list --resource-group {rg} --name {namespacename}').get_output_in_json()
        self.assertEqual(len(iprule['ipRules']), 2)

        # Remove IPRule
        iprule = self.cmd(
            'eventhubs namespace network-rule remove --resource-group {rg} --name {namespacename} --ip-address {ipmask2}').get_output_in_json()
        self.assertEqual(len(iprule['ipRules']), 1)
        self.assertTrue(iprule['ipRules'][0]['ipMask'] == '1.1.1.1')

        # add vnetrule
        vnetrule = self.cmd(
            'eventhubs namespace network-rule add --resource-group {rg} --name {namespacename} --subnet ' +
            created_subnet1['id'] + ' --ignore-missing-endpoint True').get_output_in_json()
        self.assertEqual(len(vnetrule['virtualNetworkRules']), 1)

        # add vnetrule2
        vnetrule = self.cmd(
            'eventhubs namespace network-rule add --resource-group {rg} --name {namespacename} --subnet ' +
            created_subnet2['id'] + ' --ignore-missing-endpoint True').get_output_in_json()
        self.assertEqual(len(vnetrule['virtualNetworkRules']), 2)

        # list Vnetrules
        self.cmd(
            'eventhubs namespace network-rule list --resource-group {rg} --name {namespacename}')

        # remove Vnetrule
        vnetrule = self.cmd(
            'eventhubs namespace network-rule remove --resource-group {rg} --name {namespacename} --subnet ' +
            created_subnet2['id']).get_output_in_json()
        self.assertEqual(len(vnetrule['virtualNetworkRules']), 1)

        # Delete Namespace list by ResourceGroup
        self.cmd('eventhubs namespace delete --resource-group {rg} --name {namespacename}')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.util import CLIError

from azure.cli.core import AzCommandsLoader
from azure.cli.core._help_loaders import HelpLoaderV1


# region TestCommandLoader
class TestCommandLoader(AzCommandsLoader):

    def __init__(self, cli_ctx=None):
        from azure.cli.core.commands import CliCommandType
        compute_custom = CliCommandType(
            operations_tmpl='{}#{{}}'.format(__name__),
        )
        super(TestCommandLoader, self).__init__(cli_ctx=cli_ctx,
                                                custom_command_type=compute_custom)
        self.cmd_to_loader_map = {}

    def load_command_table(self, args):
        with self.command_group('test') as g:
            g.custom_command('alpha', 'dummy_handler')

        return self.command_table

    def load_arguments(self, command):
        with self.argument_context('test') as c:
            c.argument('arg1', options_list=['--arg1', '-a'])
            c.argument('arg2', options_list=['--arg2', '-b'], help="Help From code.")
        with self.argument_context('test alpha') as c:
            c.positional('arg4', metavar="ARG4")
        self._update_command_definitions()  # pylint: disable=protected-access


def dummy_handler(arg1, arg2=None, arg3=None, arg4=None):
    """
    Short summary here. Long summary here. Still long summary.
    :param arg1: arg1's docstring help text
    :param arg2: arg2's docstring help text
    :param arg3: arg3's docstring help text
    :param arg4: arg4's docstring help text
    """
    pass


COMMAND_LOADER_CLS = TestCommandLoader

# region Test Help Loader


class JsonLoaderMixin(object):
    """A class containing helper methods for Json Loaders."""

    # get the list of json help file names for the command or group
    @staticmethod
    def _get_json_help_files_list(nouns, cmd_loader_map_ref):
        import inspect
        import os

        command_nouns = " ".join(nouns)
        # if command in map, get the loader. Path of loader is path of helpfile.
        ldr_or_none = cmd_loader_map_ref.get(command_nouns, [None])[0]
        if ldr_or_none:
            loaders = {ldr_or_none}
        else:
            loaders = set()

        # otherwise likely a group, try to find all command loaders under group as the group help could be defined
        # in either.
        if not loaders:
            for cmd_name, cmd_ldr in cmd_loader_map_ref.items():
                # if first word in loader name is the group, this is a command in the command group
                if cmd_name.startswith(command_nouns + " "):
                    loaders.add(cmd_ldr[0])

        results = []
        if loaders:
            for loader in loaders:
                loader_file_path = inspect.getfile(loader.__class__)
                dir_name = os.path.dirname(loader_file_path)
                files = os.listdir(dir_name)
                for file in files:
                    if file.endswith("help.json"):
                        help_file_path = os.path.join(dir_name, file)
                        results.append(help_file_path)
        return results

    @staticmethod
    def _parse_json_from_string(text, help_file_path):
        import os
        import json

        dir_name, base_name = os.path.split(help_file_path)
        pretty_file_path = os.path.join(os.path.basename(dir_name), base_name)

        if not text:
            raise CLIError("No content passed for {}.".format(pretty_file_path))

        try:
            return json.loads(text)
        except ValueError as e:
            raise CLIError("Error parsing {}:\n\n{}".format(pretty_file_path, e))


# test Help Loader, loads from help.json
class DummyHelpLoader(HelpLoaderV1, JsonLoaderMixin):
    # This loader has different keys in the data object. Except for "arguments" and "examples".
    core_attrs_to_keys = [("short_summary", "short"), ("long_summary", "long")]
    body_attrs_to_keys = core_attrs_to_keys + [("links", "hyper-links")]
    param_attrs_to_keys = core_attrs_to_keys + [("value_sources", "sources")]

    @property
    def version(self):
        return 2

    def get_noun_help_file_names(self, nouns):
        cmd_loader_map_ref = self.help_ctx.cli_ctx.invocation.commands_loader.cmd_to_loader_map
        return self._get_json_help_files_list(nouns, cmd_loader_map_ref)

    def update_file_contents(self, file_contents):
        for file_name in file_contents:
            if file_name not in self._file_content_dict:
                data_dict = {file_name: self._parse_json_from_string(file_contents[file_name], file_name)}
                self._file_content_dict.update(data_dict)

    def load_entry_data(self, help_obj, parser):
        prog = parser.prog if hasattr(parser, "prog") else parser._prog_prefix  # pylint: disable=protected-access
        command_nouns = prog.split()[1:]
        cmd_loader_map_ref = self.help_ctx.cli_ctx.invocation.commands_loader.cmd_to_loader_map

        files_list = self._get_json_help_files_list(command_nouns, cmd_loader_map_ref)
        data_list = [self._file_content_dict[name] for name in files_list]

        self._entry_data = self._get_entry_data(help_obj.command, data_list)

    def load_help_body(self, help_obj):
        self._update_obj_from_data_dict(help_obj, self._entry_data, self.body_attrs_to_keys)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

try:
    import xmlrpclib
except ImportError:
    import xmlrpc.client as xmlrpclib  # pylint: disable=import-error


def is_available_on_pypi(module_name, module_version):
    client = xmlrpclib.ServerProxy('https://pypi.python.org/pypi')
    available_versions = client.package_releases(module_name, True)
    return module_version in available_versions



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json
import unittest
import mock
import requests

from azure.cli.command_modules.find.custom import (Example, call_aladdin_service,
                                                   get_generated_examples, clean_from_http_answer)


def create_valid_http_response():
    mock_response = requests.Response()
    mock_response.status_code = 200
    data = [{
        'title': 'RunTestAutomation',
        'snippet': 'az find'
    }, {
        'title': 'az test',
        'snippet': 'The title'
    }]
    mock_response._content = json.dumps(data)
    return mock_response


def create_empty_http_response():
    mock_response = requests.Response()
    mock_response.status_code = 200
    data = []
    mock_response._content = json.dumps(data)
    return mock_response


class FindCustomCommandTest(unittest.TestCase):

    def test_call_aladdin_service(self):
        mock_response = create_valid_http_response()

        with mock.patch('requests.get', return_value=(mock_response)):
            response = call_aladdin_service('RunTestAutomation')
            self.assertEqual(200, response.status_code)
            self.assertEqual(2, len(json.loads(response.content)))

    def test_example_clean_from_http_answer(self):
        cleaned_responses = []
        mock_response = create_valid_http_response()

        for response in json.loads(mock_response.content):
            cleaned_responses.append(clean_from_http_answer(response))

        self.assertEqual('RunTestAutomation', cleaned_responses[0].title)
        self.assertEqual('az find', cleaned_responses[0].snippet)
        self.assertEqual('The title', cleaned_responses[1].title)
        self.assertEqual('az test', cleaned_responses[1].snippet)

    def test_get_generated_examples_full(self):
        examples = []
        mock_response = create_valid_http_response()

        with mock.patch('requests.get', return_value=(mock_response)):
            examples = get_generated_examples('RunTestAutomation')

            self.assertEqual('RunTestAutomation', examples[0].title)
            self.assertEqual('az find', examples[0].snippet)
            self.assertEqual('The title', examples[1].title)
            self.assertEqual('az test', examples[1].snippet)

    def test_get_generated_examples_empty(self):
        examples = []
        mock_response = create_empty_http_response()

        with mock.patch('requests.get', return_value=(mock_response)):
            examples = get_generated_examples('RunTestAutomation')

            self.assertEqual(0, len(examples))


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=unused-argument, line-too-long

from msrestazure.tools import is_valid_resource_id, parse_resource_id  # pylint: disable=import-error
from knack.log import get_logger
from azure.cli.core.profiles import ResourceType
from azure.cli.core.commands.client_factory import get_subscription_id
from azure.cli.core.util import CLIError
from ._client_factory import resource_client_factory, network_client_factory

logger = get_logger(__name__)
DEFAULT_VNET_ADDRESS_PREFIX = '10.0.0.0/16'
DEFAULT_SUBNET_PREFIX = '10.0.0.0/24'


# pylint: disable=too-many-locals, too-many-statements
def prepare_vnet(cmd, server_name, vnet, subnet, resource_group_name, loc, delegation_service_name, vnet_address_pref, subnet_address_pref):
    Delegation, Subnet, VirtualNetwork, AddressSpace = cmd.get_models('Delegation', 'Subnet', 'VirtualNetwork',
                                                                      'AddressSpace',
                                                                      resource_type=ResourceType.MGMT_NETWORK)
    delegation = Delegation(name=delegation_service_name, service_name=delegation_service_name)
    nw_client = network_client_factory(cmd.cli_ctx)

    # pylint: disable=too-many-nested-blocks
    if subnet is not None and vnet is None:
        if is_valid_resource_id(subnet):
            logger.warning("You have supplied a Subnet Id. Verifying its existence...")
            parsed_subnet_id = parse_resource_id(subnet)
            subnet_name = parsed_subnet_id['resource_name']
            vnet_name = parsed_subnet_id['name']
            resource_group = parsed_subnet_id['resource_group']
            subscription = parsed_subnet_id['subscription']
            resource_client = resource_client_factory(cmd.cli_ctx)
            rg = resource_client.resource_groups.get(resource_group)
            location = rg.location
            validate_rg_loc_sub(subscription, location, get_subscription_id(cmd.cli_ctx), loc)
            subnet_result = check_resource_existence(cmd, subnet_name, vnet_name, resource_group)
            if subnet_result:
                logger.info('Using existing subnet "%s" in resource group "%s"', subnet_result.name, resource_group)

                if not subnet_result.delegations:
                    logger.info('Adding "%s" delegation to the existing subnet.', )
                    subnet_result.delegations = [delegation]
                    subnet_result = nw_client.subnets.begin_create_or_update(resource_group, vnet_name, subnet_name,
                                                                             subnet_result).result()
                else:
                    for delgtn in subnet_result.delegations:
                        if delgtn.service_name != delegation_service_name:
                            raise CLIError("Can not use subnet with existing delegations other than {}".format(
                                delegation_service_name))
            else:
                logger.warning("The supplied subnet id does not exist.")
                subnet_result = _create_vnet_subnet_delegation(nw_client, resource_group_name, vnet_name,
                                                               'Subnet' + server_name[6:], location, server_name,
                                                               delegation,
                                                               VirtualNetwork, Subnet, AddressSpace,
                                                               DEFAULT_VNET_ADDRESS_PREFIX, DEFAULT_SUBNET_PREFIX)
        else:
            raise CLIError("Incorrectly formed Subnet id.")
    elif subnet is None and vnet is not None:
        if is_valid_resource_id(vnet):
            logger.warning("You have supplied a Vnet Id. Verifying its existence...")
            parsed_vnet_id = parse_resource_id(vnet)
            vnet_name = parsed_vnet_id['resource_name']
            resource_group = parsed_vnet_id['resource_group']
            subscription = parsed_vnet_id['subscription']
            resource_client = resource_client_factory(cmd.cli_ctx)
            rg = resource_client.resource_groups.get(resource_group)
            location = rg.location
            validate_rg_loc_sub(subscription, location, get_subscription_id(cmd.cli_ctx), loc)
            subnet_result = _create_vnet_subnet_delegation(nw_client, resource_group, vnet_name, 'Subnet' + server_name[6:],
                                                           location, server_name, delegation, VirtualNetwork, Subnet,
                                                           AddressSpace, DEFAULT_VNET_ADDRESS_PREFIX,
                                                           DEFAULT_SUBNET_PREFIX)
        elif len(vnet.split('\\')) == 1:
            logger.warning("You have supplied a Vnet Name. Verifying its existence...")
            subnet_result = _create_vnet_subnet_delegation(nw_client, resource_group_name, vnet, 'Subnet' + server_name[6:],
                                                           loc, server_name, delegation, VirtualNetwork, Subnet,
                                                           AddressSpace, DEFAULT_VNET_ADDRESS_PREFIX,
                                                           DEFAULT_SUBNET_PREFIX)

            # If the supplied Vnet name was an existing one, ensure that it was on the right sub, vnet and location.
            parsed_subnet_id = parse_resource_id(subnet_result.id)
            resource_group = parsed_subnet_id['resource_group']
            subscription = parsed_subnet_id['subscription']
            resource_client = resource_client_factory(cmd.cli_ctx)
            rg = resource_client.resource_groups.get(resource_group)
            location = rg.location
            validate_rg_loc_sub(subscription, location, get_subscription_id(cmd.cli_ctx), loc)
        else:
            raise CLIError("Incorrectly formed Vnet id or Vnet name")
    elif subnet is not None and vnet is not None and vnet_address_pref is not None and subnet_address_pref is not None:
        if (len(vnet.split('\\')) == 1) and (len(subnet.split('\\')) == 1):
            subnet_result = _create_with_resource_names(cmd, vnet, subnet, resource_group_name, delegation, nw_client,
                                                        delegation_service_name, server_name, VirtualNetwork, Subnet,
                                                        AddressSpace, loc, vnet_address_pref,
                                                        subnet_address_pref)
        else:
            raise CLIError(
                "If you pass an address prefix, please consider passing a name (instead of Id) for a subnet or vnet.")
    elif subnet is not None and vnet is not None:
        if (len(vnet.split('\\')) == 1) and (len(subnet.split('\\')) == 1):
            subnet_result = _create_with_resource_names(cmd, vnet, subnet, resource_group_name, delegation, nw_client,
                                                        delegation_service_name, server_name, VirtualNetwork, Subnet,
                                                        AddressSpace, loc, DEFAULT_VNET_ADDRESS_PREFIX,
                                                        DEFAULT_SUBNET_PREFIX)
        else:
            raise CLIError("If you pass both --vnet and --subnet, consider passing names instead of ids.")
    else:
        return None
    return subnet_result.id


def _create_with_resource_names(cmd, vnet, subnet, resource_group_name, delegation, nw_client,
                                delegation_service_name, server_name, VirtualNetwork, Subnet,
                                AddressSpace, loc, vnet_add, subnet_add):
    if (len(vnet.split('\\')) == 1) and (len(subnet.split('\\')) == 1):
        subnet_result = check_resource_existence(cmd, subnet, vnet, resource_group_name)
        # pylint: disable=no-else-return
        # disabling no-else-return as both if and else return or raise a CLI error, which is needed
        if subnet_result:
            logger.info('Using existing subnet "%s" in resource group "%s"', subnet_result.name, resource_group_name)

            if not subnet_result.delegations:
                logger.info('Adding "%s" delegation to the existing subnet.', )
                subnet_result.delegations = [delegation]
                subnet_result = nw_client.subnets.begin_create_or_update(resource_group_name, vnet, subnet,
                                                                         subnet_result).result()
            else:
                for delgtn in subnet_result.delegations:
                    if delgtn.service_name != delegation_service_name:
                        raise CLIError("Can not use subnet with existing delegations other than {}".format(
                            delegation_service_name))
            return subnet_result
        else:
            logger.warning(
                "The Subnet does not exist. Checking the existence of the Vnet...")
            return _create_vnet_subnet_delegation(nw_client, resource_group_name, vnet, subnet, loc,
                                                  server_name, delegation, VirtualNetwork, Subnet,
                                                  AddressSpace, vnet_add,
                                                  subnet_add)
    else:
        raise CLIError("Invalid Vnet Name or Subnet Name provided.")


def validate_rg_loc_sub(s_subscription, s_location, subscription, location):
    if not ((s_location == location) and (s_subscription == subscription)):
        raise CLIError(
            "Incorrect Usage : The location and subscription of the server,Vnet and Subnet should be same.")


def check_resource_existence(cmd, subnet_name, vnet_name, resource_group_name, ):
    nw_client = network_client_factory(cmd.cli_ctx)
    subnet = _get_resource(nw_client.subnets, resource_group_name, vnet_name, subnet_name)
    return subnet


def _create_vnet_subnet_delegation(nw_client, resource_group, vnet_name, subnet_name, location, server_name, delegation,
                                   VirtualNetwork, Subnet, AddressSpace, vnet_address_pref, subnet_address_pref):
    from azure.core.exceptions import HttpResponseError
    try:
        vnet_exist = _get_resource(nw_client.virtual_networks, resource_group, vnet_name)
        if not vnet_exist:
            logger.info('The Vnet does not exist. Creating new vnet "%s" in resource group "%s"',
                        vnet_name, resource_group)
            nw_client.virtual_networks.begin_create_or_update(resource_group,
                                                              vnet_name,
                                                              VirtualNetwork(name=vnet_name,
                                                                             location=location,
                                                                             address_space=AddressSpace(
                                                                                 address_prefixes=[
                                                                                     vnet_address_pref])))
        subnet_result = Subnet(
            name=subnet_name,
            location=location,
            address_prefix=subnet_address_pref,
            delegations=[delegation])

        logger.info('Creating new subnet "%s" in resource group "%s"', subnet_name, resource_group)
        return nw_client.subnets.begin_create_or_update(resource_group, vnet_name, subnet_name,
                                                        subnet_result).result()
    except HttpResponseError as err:
        if err.error.code == 'NetcfgInvalidSubnet':
            raise CLIError('Cannot add the subnet {} to the vnet {}.The subnet address space exceeds'
                           ' the available vnet address space.'.format(subnet_name, vnet_name))
        raise


def _get_resource(client, resource_group_name, *subresources):
    from azure.core.exceptions import HttpResponseError
    try:
        resource = client.get(resource_group_name, *subresources)
        return resource
    except HttpResponseError as ex:
        if ex.error.code == "NotFound" or ex.error.code == "ResourceNotFound":
            return None
        raise


def create_vnet(cmd, servername, location, resource_group_name, delegation_service_name):
    Subnet, VirtualNetwork, AddressSpace, Delegation = cmd.get_models('Subnet', 'VirtualNetwork', 'AddressSpace',
                                                                      'Delegation',
                                                                      resource_type=ResourceType.MGMT_NETWORK)
    client = network_client_factory(cmd.cli_ctx)
    vnet_name, subnet_name, vnet_address_prefix, subnet_prefix = _create_vnet_metadata(servername[6:])

    logger.warning('Creating new vnet "%s" in resource group "%s"...', vnet_name, resource_group_name)
    client.virtual_networks.begin_create_or_update(resource_group_name, vnet_name,
                                                   VirtualNetwork(name=vnet_name, location=location,
                                                                  address_space=AddressSpace(
                                                                      address_prefixes=[vnet_address_prefix])))
    delegation = Delegation(name=delegation_service_name, service_name=delegation_service_name)
    subnet = Subnet(name=subnet_name, location=location, address_prefix=subnet_prefix, delegations=[delegation])

    logger.warning('Creating new subnet "%s" in resource group "%s" and delegating it to "%s"...', subnet_name,
                   resource_group_name, delegation_service_name)
    subnet = client.subnets.begin_create_or_update(resource_group_name, vnet_name, subnet_name, subnet).result()
    return subnet.id


def _create_vnet_metadata(servername):
    vnet_name = 'VNET' + servername
    subnet_name = 'Subnet' + servername
    vnet_address_prefix = DEFAULT_VNET_ADDRESS_PREFIX
    subnet_prefix = DEFAULT_SUBNET_PREFIX
    return vnet_name, subnet_name, vnet_address_prefix, subnet_prefix



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.util import CLIError

ROLE_COMMAND_MAX_RETRY = 20
ROLE_COMMAND_SLEEP_DURATION = 10


def retry(func, sleep_duration=ROLE_COMMAND_SLEEP_DURATION, max_retry=ROLE_COMMAND_MAX_RETRY):
    """Retry func until success."""
    # Due to unstable role definition ARIs: https://github.com/Azure/azure-cli/issues/3187
    import time
    while True:
        try:
            return func()
        except (AssertionError, CLIError):
            # AssertionError is raised by checks in self.cmd or self.assert*
            # CLIError is raised by failed command execution
            if max_retry > 0:
                max_retry -= 1
                time.sleep(sleep_duration)
            else:
                raise


def escape_apply_kwargs(val):
    """Replace {} as {{}} so that val is preserved after _apply_kwargs."""
    return val.replace('{', "{{").replace('}', "}}")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure_devtools.scenario_tests import RecordingProcessor, mock_in_unit_test


MOCK_JOB_ID = '00000000-0000-0000-0000-000000000000'


class JobIdReplacer(RecordingProcessor):

    """Replace the random job id with a fixed mock name."""
    def process_request(self, request):
        import re
        request.uri = re.sub('/Jobs/([^/?]+)', '/Jobs/{}'.format(MOCK_JOB_ID), request.uri)
        return request

    def process_response(self, response):
        if response['body']['string']:
            response['body']['string'] = self._replace_job_id(response['body']['string'])
        return response

    # pylint: disable=no-self-use
    def _replace_job_id(self, val):
        import re
        if 'jobId' in val:
            return re.sub(r'"jobId":"([^"]+)"', r'"jobId":"{}"'.format(MOCK_JOB_ID), val, flags=re.IGNORECASE)
        return val


def patch_uuid_str(unit_test):

    def _mock_get_uuid_str(*args, **kwargs):  # pylint: disable=unused-argument
        return MOCK_JOB_ID

    mock_in_unit_test(unit_test, 'azure.cli.command_modules.dla.custom._get_uuid_str', _mock_get_uuid_str)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from __future__ import print_function

import logging
import shutil
import inspect
from inspect import getmembers as inspect_getmembers

import unittest
import mock
import tempfile

from knack.help import GroupHelpFile, HelpAuthoringException
from azure.cli.core._help import CliCommandHelpFile

from azure.cli.core.mock import DummyCli
from azure.cli.core.commands import _load_command_loader
from azure.cli.core.file_util import get_all_help

logger = logging.getLogger(__name__)

# Command loader module
MOCKED_COMMAND_LOADER_MOD = "test_help_loaders"


# mock command loader method so that only the mock command loader can be loaded.
def mock_load_command_loader(loader, args, name, prefix):
    return _load_command_loader(loader, args, name, "azure.cli.core.tests.")


# mock inspect.getfile to discover directory containing help files.
def get_mocked_inspect_getfile(expected_arg, return_value):

    def inspect_getfile(obj):
        if obj == expected_arg:
            return return_value
        else:
            return inspect.getfile(obj)
    return inspect_getfile


# mock getmembers so test help loader can be injected into AzCliHelp class' versioned loader list
def mock_inspect_getmembers(object, predicate=None):
    import azure.cli.core.tests.test_help_loaders as possible_loaders

    if "azure.cli.core._help_loaders" in repr(object) and "is_loader_cls" in repr(predicate):
        result = inspect_getmembers(object, predicate)
        result.extend(inspect_getmembers(possible_loaders, predicate))
        return list(set(result))
    else:
        return inspect_getmembers(object, predicate)


def _store_parsers(parser, d):
    for s in parser.subparsers.values():
        d[_get_parser_name(s)] = s
        if _is_group(s):
            for c in s.choices.values():
                d[_get_parser_name(c)] = c
                _store_parsers(c, d)


def _is_group(parser):
    return getattr(parser, 'choices', None) is not None


def _get_parser_name(parser):
    # pylint:disable=protected-access
    return (parser._prog_prefix if hasattr(parser, '_prog_prefix') else parser.prog)[len('az '):]


def create_invoker_and_load_cmds_and_args(cli_ctx):
    from knack import events
    from azure.cli.core.commands import register_cache_arguments
    from azure.cli.core.commands.arm import register_global_subscription_argument, register_ids_argument

    invoker = cli_ctx.invocation_cls(cli_ctx=cli_ctx, commands_loader_cls=cli_ctx.commands_loader_cls,
                                     parser_cls=cli_ctx.parser_cls, help_cls=cli_ctx.help_cls)
    cli_ctx.invocation = invoker
    invoker.commands_loader.skip_applicability = True
    invoker.commands_loader.load_command_table(None)

    # Deal with failed import MainCommandsLoader.load_command_table._update_command_table_from_modules
    # during tox test.
    if not invoker.commands_loader.cmd_to_loader_map:
        module_command_table, module_group_table = mock_load_command_loader(invoker.commands_loader, None,
                                                                            MOCKED_COMMAND_LOADER_MOD, None)
        for cmd in module_command_table.values():
            cmd.command_source = MOCKED_COMMAND_LOADER_MOD
        invoker.commands_loader.command_table.update(module_command_table)
        invoker.commands_loader.command_group_table.update(module_group_table)

    # turn off applicability check for all loaders
    for loaders in invoker.commands_loader.cmd_to_loader_map.values():
        for loader in loaders:
            loader.skip_applicability = True

    for command in invoker.commands_loader.command_table:
        invoker.commands_loader.load_arguments(command)

    register_global_subscription_argument(cli_ctx)
    register_ids_argument(cli_ctx)  # global subscription must be registered first!
    register_cache_arguments(cli_ctx)
    cli_ctx.raise_event(events.EVENT_INVOKER_POST_CMD_TBL_CREATE, commands_loader=invoker.commands_loader)
    invoker.parser.load_command_table(invoker.commands_loader)


# TODO update this CLASS to properly load all help....
class HelpTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # Ensure initialization has occurred correctly
        logging.basicConfig(level=logging.DEBUG)

    @classmethod
    def tearDownClass(cls):
        logging.shutdown()

    def test_help_loads(self):
        from azure.cli.core.commands.arm import register_global_subscription_argument, register_ids_argument
        import knack.events as events

        parser_dict = {}
        cli = DummyCli()
        help_ctx = cli.help_cls(cli)
        try:
            cli.invoke(['-h'])
        except SystemExit:
            pass
        cmd_tbl = cli.invocation.commands_loader.command_table
        cli.invocation.parser.load_command_table(cli.invocation.commands_loader)
        for cmd in cmd_tbl:
            try:
                cmd_tbl[cmd].loader.command_name = cmd
                cmd_tbl[cmd].loader.load_arguments(cmd)
            except KeyError:
                pass
        cli.register_event(events.EVENT_INVOKER_POST_CMD_TBL_CREATE, register_global_subscription_argument)
        cli.register_event(events.EVENT_INVOKER_POST_CMD_TBL_CREATE, register_ids_argument)
        cli.raise_event(events.EVENT_INVOKER_CMD_TBL_LOADED, command_table=cmd_tbl)
        cli.invocation.parser.load_command_table(cli.invocation.commands_loader)
        _store_parsers(cli.invocation.parser, parser_dict)

        # TODO: do we want to update this as it doesn't actually load all help.
        # We do have a CLI linter which does indeed load all help.
        for name, parser in parser_dict.items():
            try:
                help_file = GroupHelpFile(help_ctx, name, parser) if _is_group(parser) \
                    else CliCommandHelpFile(help_ctx, name, parser)
                help_file.load(parser)
            except Exception as ex:
                raise HelpAuthoringException('{}, {}'.format(name, ex))


class TestHelpLoads(unittest.TestCase):
    def setUp(self):
        from knack.help_files import helps
        self.test_cli = DummyCli()
        self.helps = helps
        self._tempdirName = tempfile.mkdtemp(prefix="help_test_temp_dir_")

    def tearDown(self):
        # delete temporary directory to be used for temp files.
        shutil.rmtree(self._tempdirName)
        self.helps.clear()

    def set_help_py(self):
        self.helps['test'] = """
            type: group
            short-summary: Foo Bar Group
            long-summary: Foo Bar Baz Group is a fun group
        """

        self.helps['test alpha'] = """
            type: command
            short-summary: Foo Bar Command
            long-summary: Foo Bar Baz Command is a fun command
            parameters:
                - name: --arg1 -a
                  short-summary: A short summary
                  populator-commands:
                  - az foo bar
                  - az bar baz
                - name: ARG4 # Note: positional's are discouraged in the CLI.
                  short-summary: Positional parameter. Not required
            examples:
                - name: Alpha Example
                  text: az test alpha --arg1 a --arg2 b --arg3 c
                  supported-profiles: 2018-03-01-hybrid, latest
                - name: A simple example unsupported on latest
                  text: az test alpha --arg1 a --arg2 b
                  unsupported-profiles: 2017-03-09-profile
        """

    def set_help_yaml(self):
        yaml_help = """
        version: 1
        content:
        - group:
            name: test
            summary: Group yaml summary
            description: Group yaml description. A.K.A long description
            links:
                - title: Azure Test Docs
                  url: "https://docs.microsoft.com/azure/test"
                - url: "https://aka.ms/just-a-url"
        - command:
            name: test alpha
            summary: Command yaml summary
            description: Command yaml description. A.K.A long description
            links:
                - title: Azure Test Alpha Docs
                  url: "https://docs.microsoft.com/azure/test/alpha"
                - url: "https://aka.ms/just-a-long-url"
            arguments:
                - name: --arg2 # we do not specify the short option in the name.
                  summary: Arg 2's summary
                  description: A true description of this parameter.
                  value-sources:
                    - string: "Number range: -5.0 to 5.0"
                    - link:
                        url: https://www.foo.com
                        title: foo
                    - link:
                        command: az test show
                        title: Show test details
                - name: ARG4  # Note: positional's are discouraged in the CLI.
                  summary: Arg4's summary, yaml. Positional arg, not required
            examples:
                - summary: A simple example
                  description: More detail on the simple example.
                  command: az test alpha --arg1 apple --arg2 ball --arg3 cat
                  supported-profiles: 2018-03-01-hybrid, latest
                - summary: Another example unsupported on latest
                  description: More detail on the unsupported example.
                  command: az test alpha --arg1 apple --arg2 ball
                  unsupported-profiles: 2017-03-09-profile
        """
        return self._create_new_temp_file(yaml_help, suffix="help.yaml")

    def set_help_json(self):
        json_help = """
            {
                "version": 2,
                "content": [
                    {
                        "group": {
                            "name": "test",
                            "short": "Group json summary",
                            "long": "Group json description. A.K.A long description",
                            "hyper-links": [
                                {
                                    "title": "Azure Json Test Docs",
                                    "url": "https://docs.microsoft.com/azure/test"
                                },
                                {
                                    "url": "https://aka.ms/just-a-url"
                                }
                            ]
                        }
                    },
                    {
                        "command": {
                            "name": "test alpha",
                            "short": "Command json summary",
                            "long": "Command json description. A.K.A long description",
                            "hyper-links": [
                                {
                                    "title": "Azure Json Test Alpha Docs",
                                    "url": "https://docs.microsoft.com/azure/test/alpha"
                                },
                                {
                                    "url": "https://aka.ms/just-a-long-url"
                                }
                            ],
                            "arguments": [
                                {
                                    "name": "--arg3",
                                    "short": "Arg 3's json summary",
                                    "long": "A truly true description of this parameter.",
                                    "sources": [
                                        {
                                            "string": "Number range: 0 to 10"
                                        },
                                        {
                                            "link": {
                                                "url": "https://www.foo-json.com",
                                                "title": "foo-json"
                                            }
                                        },
                                        {
                                            "link": {
                                                "command": "az test show",
                                                "title": "Show test details. Json file"
                                            }
                                        }
                                    ]
                                },
                                {
                                    "name": "ARG4",
                                    "summary": "Arg4's summary, json. Positional arg, not required"
                                }
                            ],
                            "examples": [
                                {
                                    "summary": "A simple example from json",
                                    "description": "More detail on the simple example.",
                                    "command": "az test alpha --arg1 alpha --arg2 beta --arg3 chi",
                                    "supported-profiles": "2018-03-01-hybrid, latest"
                                }
                            ]
                        }
                    }
                ]
            }
        """
        return self._create_new_temp_file(json_help, suffix="help.json")

    # Mock logic in core.MainCommandsLoader.load_command_table for retrieving installed modules.
    @mock.patch('pkgutil.iter_modules', side_effect=lambda x: [(None, MOCKED_COMMAND_LOADER_MOD, None)])
    @mock.patch('azure.cli.core.commands._load_command_loader', side_effect=mock_load_command_loader)
    def test_basic(self, mocked_load, mocked_pkg_util):
        with self.assertRaises(SystemExit):
            self.test_cli.invoke(["test", "alpha", "-h"])

    # Mock logic in core.MainCommandsLoader.load_command_table for retrieving installed modules.
    @mock.patch('pkgutil.iter_modules', side_effect=lambda x: [(None, MOCKED_COMMAND_LOADER_MOD, None)])
    @mock.patch('azure.cli.core.commands._load_command_loader', side_effect=mock_load_command_loader)
    def test_load_from_help_py(self, mocked_load, mocked_pkg_util):
        self.set_help_py()
        create_invoker_and_load_cmds_and_args(self.test_cli)
        group_help_obj = next((help for help in get_all_help(self.test_cli) if help.command == "test"), None)
        command_help_obj = next((help for help in get_all_help(self.test_cli) if help.command == "test alpha"), None)

        # Test that group and command help are successfully displayed.
        with self.assertRaises(SystemExit):
            self.test_cli.invoke(["test", "-h"])
        with self.assertRaises(SystemExit):
            self.test_cli.invoke(["test", "alpha", "-h"])

        # Test group help
        self.assertIsNotNone(group_help_obj)
        self.assertEqual(group_help_obj.short_summary, "Foo Bar Group.")
        self.assertEqual(group_help_obj.long_summary, "Foo Bar Baz Group is a fun group.")

        # Test command help
        self.assertIsNotNone(command_help_obj)
        self.assertEqual(command_help_obj.short_summary, "Foo Bar Command.")
        self.assertEqual(command_help_obj.long_summary, "Foo Bar Baz Command is a fun command.")

        # test that parameters and help are loaded from command function docstring, argument registry help and help.py
        obj_param_dict = {param.name: param for param in command_help_obj.parameters}
        param_name_set = {"--arg1 -a", "--arg2 -b", "--arg3", "ARG4"}
        self.assertTrue(set(obj_param_dict.keys()).issuperset(param_name_set))

        self.assertEqual(obj_param_dict["--arg3"].short_summary, "Arg3's docstring help text.")
        self.assertEqual(obj_param_dict["ARG4"].short_summary, "Positional parameter. Not required.")
        self.assertEqual(obj_param_dict["--arg1 -a"].short_summary, "A short summary.")

        self.assertEqual(obj_param_dict["--arg1 -a"].short_summary, "A short summary.")
        self.assertEqual(obj_param_dict["--arg1 -a"].value_sources[0]['link']['command'], "az foo bar")
        self.assertEqual(obj_param_dict["--arg1 -a"].value_sources[1]['link']['command'], "az bar baz")

        if self.test_cli.cloud.profile in ['2018-03-01-hybrid', 'latest']:
            self.assertEqual(command_help_obj.examples[0].short_summary, "Alpha Example")
            self.assertEqual(command_help_obj.examples[0].command, "az test alpha --arg1 a --arg2 b --arg3 c")
            self.assertEqual(command_help_obj.examples[0].supported_profiles, "2018-03-01-hybrid, latest")
            self.assertEqual(command_help_obj.examples[0].unsupported_profiles, None)

            self.assertEqual(command_help_obj.examples[1].supported_profiles, None)
            self.assertEqual(command_help_obj.examples[1].unsupported_profiles, "2017-03-09-profile")

        if self.test_cli.cloud.profile == '2019-03-01-hybrid':
            self.assertEqual(len(command_help_obj.examples), 1)
            self.assertEqual(command_help_obj.examples[0].short_summary, "A simple example unsupported on latest")
            self.assertEqual(command_help_obj.examples[0].command, "az test alpha --arg1 a --arg2 b")
            self.assertEqual(command_help_obj.examples[0].unsupported_profiles, '2017-03-09-profile')

        if self.test_cli.cloud.profile == '2017-03-09-profile':
            self.assertEqual(len(command_help_obj.examples), 0)

    @mock.patch('pkgutil.iter_modules', side_effect=lambda x: [(None, MOCKED_COMMAND_LOADER_MOD, None)])
    @mock.patch('azure.cli.core.commands._load_command_loader', side_effect=mock_load_command_loader)
    def test_load_from_help_yaml(self, mocked_load, mocked_pkg_util):
        # setup help.py and help.yaml help.
        self.set_help_py()
        yaml_path = self.set_help_yaml()
        create_invoker_and_load_cmds_and_args(self.test_cli)

        # mock logic in core._help_loaders for retrieving yaml file from loader path.
        expected_arg = self.test_cli.invocation.commands_loader.cmd_to_loader_map['test alpha'][0].__class__
        with mock.patch('inspect.getfile', side_effect=get_mocked_inspect_getfile(expected_arg, yaml_path)):
            group_help_obj = next((help for help in get_all_help(self.test_cli) if help.command == "test"), None)
            command_help_obj = next((help for help in get_all_help(self.test_cli) if help.command == "test alpha"), None)  # pylint: disable=line-too-long

            # Test that group and command help are successfully displayed.
            with self.assertRaises(SystemExit):
                self.test_cli.invoke(["test", "-h"])
            with self.assertRaises(SystemExit):
                self.test_cli.invoke(["test", "alpha", "-h"])

        # Test group help
        self.assertIsNotNone(group_help_obj)
        self.assertEqual(group_help_obj.short_summary, "Group yaml summary.")
        self.assertEqual(group_help_obj.long_summary, "Group yaml description. A.K.A long description.")
        self.assertEqual(group_help_obj.links[0], {"title": "Azure Test Docs", "url": "https://docs.microsoft.com/azure/test"})
        self.assertEqual(group_help_obj.links[1], {"url": "https://aka.ms/just-a-url"})

        # Test command help
        self.assertIsNotNone(command_help_obj)
        self.assertEqual(command_help_obj.short_summary, "Command yaml summary.")
        self.assertEqual(command_help_obj.long_summary, "Command yaml description. A.K.A long description.")
        self.assertEqual(command_help_obj.links[0], {"title": "Azure Test Alpha Docs",
                                                     "url": "https://docs.microsoft.com/azure/test/alpha"})
        self.assertEqual(command_help_obj.links[1], {"url": "https://aka.ms/just-a-long-url"})

        # test that parameters and help are loaded from command function docstring, argument registry help and help.yaml
        obj_param_dict = {param.name: param for param in command_help_obj.parameters}
        param_name_set = {"--arg1 -a", "--arg2 -b", "--arg3", "ARG4"}
        self.assertTrue(set(obj_param_dict.keys()).issuperset(param_name_set))

        self.assertEqual(obj_param_dict["--arg1 -a"].short_summary, "A short summary.")
        self.assertEqual(obj_param_dict["--arg3"].short_summary, "Arg3's docstring help text.")
        self.assertEqual(obj_param_dict["ARG4"].short_summary, "Arg4's summary, yaml. Positional arg, not required.")

        self.assertEqual(obj_param_dict["--arg2 -b"].short_summary, "Arg 2's summary.")
        self.assertEqual(obj_param_dict["--arg2 -b"].long_summary, "A true description of this parameter.")
        self.assertEqual(obj_param_dict["--arg2 -b"].value_sources[0], {"string": "Number range: -5.0 to 5.0"})
        self.assertEqual(obj_param_dict["--arg2 -b"].value_sources[1]['link'], {"url": "https://www.foo.com",
                                                                                "title": "foo"})
        self.assertEqual(obj_param_dict["--arg2 -b"].value_sources[2]['link'], {"command": "az test show",
                                                                                "title": "Show test details"})

        if self.test_cli.cloud.profile in ['2018-03-01-hybrid', 'latest']:
            self.assertEqual(command_help_obj.examples[0].short_summary, "A simple example")
            self.assertEqual(command_help_obj.examples[0].long_summary, "More detail on the simple example.")
            self.assertEqual(command_help_obj.examples[0].command, "az test alpha --arg1 apple --arg2 ball --arg3 cat")
            self.assertEqual(command_help_obj.examples[0].supported_profiles, "2018-03-01-hybrid, latest")
            self.assertEqual(command_help_obj.examples[0].unsupported_profiles, None)

            self.assertEqual(command_help_obj.examples[1].supported_profiles, None)
            self.assertEqual(command_help_obj.examples[1].unsupported_profiles, "2017-03-09-profile")

        if self.test_cli.cloud.profile == '2019-03-01-hybrid':
            self.assertEqual(len(command_help_obj.examples), 1)
            self.assertEqual(command_help_obj.examples[0].short_summary, "Another example unsupported on latest")
            self.assertEqual(command_help_obj.examples[0].command, "az test alpha --arg1 apple --arg2 ball")
            self.assertEqual(command_help_obj.examples[0].unsupported_profiles, '2017-03-09-profile')

        if self.test_cli.cloud.profile == '2017-03-09-profile':
            self.assertEqual(len(command_help_obj.examples), 0)

    @mock.patch('inspect.getmembers', side_effect=mock_inspect_getmembers)
    @mock.patch('pkgutil.iter_modules', side_effect=lambda x: [(None, MOCKED_COMMAND_LOADER_MOD, None)])
    @mock.patch('azure.cli.core.commands._load_command_loader', side_effect=mock_load_command_loader)
    def test_load_from_help_json(self, mocked_load, mocked_pkg_util, mocked_getmembers):
        # setup help.py, help.yaml and help.json
        self.set_help_py()
        path = self.set_help_yaml()  # either (yaml or json) path should work. As both files are in the same temp dir.
        self.set_help_json()
        create_invoker_and_load_cmds_and_args(self.test_cli)

        # mock logic in core._help_loaders for retrieving yaml file from loader path.
        expected_arg = self.test_cli.invocation.commands_loader.cmd_to_loader_map['test alpha'][0].__class__
        with mock.patch('inspect.getfile', side_effect=get_mocked_inspect_getfile(expected_arg, path)):
            group_help_obj = next((help for help in get_all_help(self.test_cli) if help.command == "test"), None)
            command_help_obj = next((help for help in get_all_help(self.test_cli) if help.command == "test alpha"),
                                    None)

            # Test that group and command help are successfully displayed.
            with self.assertRaises(SystemExit):
                self.test_cli.invoke(["test", "-h"])
            with self.assertRaises(SystemExit):
                self.test_cli.invoke(["test", "alpha", "-h"])

        # Test group help
        self.assertIsNotNone(group_help_obj)
        self.assertEqual(group_help_obj.short_summary, "Group json summary.")
        self.assertEqual(group_help_obj.long_summary, "Group json description. A.K.A long description.")
        self.assertEqual(group_help_obj.links[0], {"title": "Azure Json Test Docs",
                                                   "url": "https://docs.microsoft.com/azure/test"})
        self.assertEqual(group_help_obj.links[1], {"url": "https://aka.ms/just-a-url"})

        # Test command help
        self.assertIsNotNone(command_help_obj)
        self.assertEqual(command_help_obj.short_summary, "Command json summary.")
        self.assertEqual(command_help_obj.long_summary, "Command json description. A.K.A long description.")
        self.assertEqual(command_help_obj.links[0], {"title": "Azure Json Test Alpha Docs",
                                                     "url": "https://docs.microsoft.com/azure/test/alpha"})
        self.assertEqual(command_help_obj.links[1], {"url": "https://aka.ms/just-a-long-url"})

        # test that parameters and help are loaded from command function docstring, argument registry help and help.yaml
        obj_param_dict = {param.name: param for param in command_help_obj.parameters}
        param_name_set = {"--arg1 -a", "--arg2 -b", "--arg3", "ARG4"}
        self.assertTrue(set(obj_param_dict.keys()).issuperset(param_name_set))

        self.assertEqual(obj_param_dict["--arg3"].short_summary, "Arg 3's json summary.")
        self.assertEqual(obj_param_dict["--arg3"].long_summary, "A truly true description of this parameter.")
        self.assertEqual(obj_param_dict["--arg3"].value_sources[0], {"string": "Number range: 0 to 10"})
        self.assertEqual(obj_param_dict["--arg3"].value_sources[1]['link'],
                         {"url": "https://www.foo-json.com", "title": "foo-json"})
        self.assertEqual(obj_param_dict["--arg3"].value_sources[2]['link'],
                         {"command": "az test show", "title": "Show test details. Json file"})

        if self.test_cli.cloud.profile in ['2018-03-01-hybrid', 'latest']:
            self.assertEqual(command_help_obj.examples[0].short_summary, "A simple example from json")
            self.assertEqual(command_help_obj.examples[0].long_summary, "More detail on the simple example.")
            self.assertEqual(command_help_obj.examples[0].command, "az test alpha --arg1 alpha --arg2 beta --arg3 chi")
            self.assertEqual(command_help_obj.examples[0].supported_profiles, "2018-03-01-hybrid, latest")

        if self.test_cli.cloud.profile == '2019-03-01-hybrid':
            # only supported example here
            self.assertEqual(len(command_help_obj.examples), 0)

        if self.test_cli.cloud.profile == '2017-03-09-profile':
            self.assertEqual(len(command_help_obj.examples), 0)

        # validate other parameters, which have help from help.py and help.yamls
        self.assertEqual(obj_param_dict["--arg1 -a"].short_summary, "A short summary.")
        self.assertEqual(obj_param_dict["--arg2 -b"].short_summary, "Arg 2's summary.")
        self.assertEqual(obj_param_dict["ARG4"].short_summary, "Arg4's summary, yaml. Positional arg, not required.")
        # arg2's help from help.yaml still preserved.
        self.assertEqual(obj_param_dict["--arg2 -b"].long_summary, "A true description of this parameter.")
        self.assertEqual(obj_param_dict["--arg2 -b"].value_sources[0], {"string": "Number range: -5.0 to 5.0"})
        self.assertEqual(obj_param_dict["--arg2 -b"].value_sources[1]['link'], {"url": "https://www.foo.com",
                                                                                "title": "foo"})
        self.assertEqual(obj_param_dict["--arg2 -b"].value_sources[2]['link'], {"command": "az test show",
                                                                                "title": "Show test details"})

    # create a temporary file in the temp dir. Return the path of the file.
    def _create_new_temp_file(self, data, suffix=""):
        with tempfile.NamedTemporaryFile(mode='w', dir=self._tempdirName, delete=False, suffix=suffix) as f:
            f.write(data)
            return f.name


class TestHelpSupportedProfiles(unittest.TestCase):
    def setUp(self):
        from azure.cli.core.profiles._shared import AZURE_API_PROFILES
        self.test_cli = DummyCli()
        self.all_profiles = AZURE_API_PROFILES
        self.all_profiles_str = " {} ".format(" , ".join(AZURE_API_PROFILES.keys()))

    def test_example_inclusion_basic(self):
        from azure.cli.core._help import CliHelpFile

        cli_ctx = self.test_cli
        cli_ctx.invocation = cli_ctx.invocation_cls(cli_ctx=cli_ctx, commands_loader_cls=cli_ctx.commands_loader_cls,
                                                    parser_cls=cli_ctx.parser_cls, help_cls=cli_ctx.help_cls)
        mock_help_file = CliHelpFile(cli_ctx.invocation.help, "")
        mock_help_file.help_ctx = cli_ctx.invocation.help
        ex_dict = {
            'summary': "Example to test supported-profiles",
            'command': "Az foo bar",
            'supported-profiles': self.all_profiles_str
        }

        # example should be included in all profiles (implicit)
        for profile in self.all_profiles.keys():
            mock_help_file.help_ctx.cli_ctx.cloud.profile = profile
            self.assertTrue(mock_help_file._should_include_example(ex_dict))

        ex_dict.update({
            'supported-profiles': self.all_profiles_str,
            'unsupported-profiles': self.all_profiles_str
        })
        # Assert that an help authoring exception is raised when both types of fields are used.
        with self.assertRaises(HelpAuthoringException):
            mock_help_file._should_include_example(ex_dict)

        del ex_dict['supported-profiles']
        del ex_dict['unsupported-profiles']

    def test_example_supported_profiles(self):
        from azure.cli.core._help import CliHelpFile

        cli_ctx = self.test_cli
        cli_ctx.invocation = cli_ctx.invocation_cls(cli_ctx=cli_ctx,
                                                    commands_loader_cls=cli_ctx.commands_loader_cls,
                                                    parser_cls=cli_ctx.parser_cls, help_cls=cli_ctx.help_cls)
        mock_help_file = CliHelpFile(cli_ctx.invocation.help, "")
        mock_help_file.help_ctx = cli_ctx.invocation.help
        ex_dict = {
            'summary': "Example to test supported-profiles",
            'command': "Az foo bar",
            'supported-profiles': self.all_profiles_str
        }

        # example should be included in all profiles (explicit)
        for profile in self.all_profiles:
            mock_help_file.help_ctx.cli_ctx.cloud.profile = profile
            self.assertTrue(mock_help_file._should_include_example(ex_dict))

        # example should be included in all filtered profiles but not excluded profile
        for excluded_profile in self.all_profiles:
            filtered_profiles = [profile for profile in self.all_profiles if profile != excluded_profile]
            ex_dict['supported-profiles'] = ", ".join(filtered_profiles)

            # excluded profile is not in supported-profiles list and example should be excluded
            mock_help_file.help_ctx.cli_ctx.cloud.profile = excluded_profile
            self.assertFalse(mock_help_file._should_include_example(ex_dict))

            # for other profiles example should show be included.
            for profile in filtered_profiles:
                mock_help_file.help_ctx.cli_ctx.cloud.profile = profile
                self.assertTrue(mock_help_file._should_include_example(ex_dict))

        # example should be included in the sole supported profile
        for sole_profile in self.all_profiles:
            mock_help_file.help_ctx.cli_ctx.cloud.profile = sole_profile

            # try different supported profiles settings in example and handle accordingly
            for profile in self.all_profiles:
                ex_dict['supported-profiles'] = " {} ".format(profile)
                should_include_example = mock_help_file._should_include_example(ex_dict)
                if profile == sole_profile:
                    self.assertTrue(should_include_example)
                else:
                    self.assertFalse(should_include_example)

    def test_example_unsupported_profiles(self):
        from azure.cli.core._help import CliHelpFile

        cli_ctx = self.test_cli
        cli_ctx.invocation = cli_ctx.invocation_cls(cli_ctx=cli_ctx,
                                                    commands_loader_cls=cli_ctx.commands_loader_cls,
                                                    parser_cls=cli_ctx.parser_cls, help_cls=cli_ctx.help_cls)
        mock_help_file = CliHelpFile(cli_ctx.invocation.help, "")
        mock_help_file.help_ctx = cli_ctx.invocation.help
        ex_dict = {
            'summary': "Example to test supported-profiles",
            'command': "Az foo bar",
            'unsupported-profiles': self.all_profiles_str
        }

        # example should be excluded from all profiles (explicit)
        for profile in self.all_profiles:
            mock_help_file.help_ctx.cli_ctx.cloud.profile = profile
            self.assertFalse(mock_help_file._should_include_example(ex_dict))

        # example should be excluded in all filtered profiles
        for included in self.all_profiles:
            filtered_profiles = [profile for profile in self.all_profiles if profile != included]
            ex_dict['unsupported-profiles'] = ", ".join(filtered_profiles)

            # included profile is not in unsupported-profiles list and example should be included
            mock_help_file.help_ctx.cli_ctx.cloud.profile = included
            self.assertTrue(mock_help_file._should_include_example(ex_dict))

            # for other profiles example should show be excluded.
            for profile in filtered_profiles:
                mock_help_file.help_ctx.cli_ctx.cloud.profile = profile
                self.assertFalse(mock_help_file._should_include_example(ex_dict))

        # example should be excluded in the sole unsupported profile
        for sole_profile in self.all_profiles:
            mock_help_file.help_ctx.cli_ctx.cloud.profile = sole_profile

            # try different unsupported profiles settings in example and handle accordingly
            for profile in self.all_profiles:
                ex_dict['unsupported-profiles'] = " {} ".format(profile)
                should_include_example = mock_help_file._should_include_example(ex_dict)
                if profile == sole_profile:
                    self.assertFalse(should_include_example)
                else:
                    self.assertTrue(should_include_example)


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import unittest
from azure.cli.command_modules.servicefabric.tests.latest.test_util import _create_cluster_with_separate_kv
from azure.cli.core.util import CLIError
from azure.cli.testsdk import ScenarioTest, LiveScenarioTest, ResourceGroupPreparer


class ServiceFabricApplicationTests(ScenarioTest):
    def _app_type_test(self):
        self.cmd('az sf application-type list -g {rg} -c {cluster_name}',
                 checks=[self.check('length(value)', 0)])
        app_type = self.cmd('az sf application-type create -g {rg} -c {cluster_name} --application-type-name {app_type_name}',
                            checks=[self.check('provisioningState', 'Succeeded')]).get_output_in_json()
        self.cmd('az sf application-type show -g {rg} -c {cluster_name} --application-type-name {app_type_name}',
                 checks=[self.check('id', app_type['id'])])
        self.cmd('az sf application-type delete -g {rg} -c {cluster_name} --application-type-name {app_type_name}')

        # SystemExit 3 'not found'
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('az sf application-type show -g {rg} -c {cluster_name} --application-type-name {app_type_name}')

    def _app_type_version_test(self):
        self.cmd('az sf application-type version list -g {rg} -c {cluster_name} --application-type-name {app_type_name}',
                 checks=[self.check('length(value)', 0)])
        app_type_version = self.cmd('az sf application-type version create -g {rg} -c {cluster_name} '
                                    '--application-type-name {app_type_name} --version {v1} --package-url {app_package_v1}',
                                    checks=[self.check('provisioningState', 'Succeeded')]).get_output_in_json()
        self.cmd('az sf application-type version show -g {rg} -c {cluster_name} --application-type-name {app_type_name} --version {v1}',
                 checks=[self.check('id', app_type_version['id'])])
        self.cmd('az sf application-type version delete -g {rg} -c {cluster_name} --application-type-name {app_type_name} --version {v1}')

        # SystemExit 3 'not found'
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('az sf application-type version show -g {rg} -c {cluster_name} --application-type-name {app_type_name} --version {v1}')

    def _app_service_test(self):
        self.cmd('az sf application list -g {rg} -c {cluster_name}',
                 checks=[self.check('length(value)', 0)])
        app = self.cmd('az sf application create -g {rg} -c {cluster_name} --application-name {app_name} '
                       '--application-type-name {app_type_name} --application-type-version {v1} --package-url {app_package_v1} '
                       '--application-parameters Mode=binary',
                       checks=[self.check('provisioningState', 'Succeeded')]).get_output_in_json()
        self.cmd('az sf application show -g {rg} -c {cluster_name} --application-name {app_name}',
                 checks=[self.check('id', app['id'])])

        service = self.cmd('az sf service create -g {rg} -c {cluster_name} --application-name {app_name} --state stateless --instance-count -1 '
                           '--service-name "{app_name}~testService" --service-type {service_type} --partition-scheme singleton',
                           checks=[self.check('provisioningState', 'Succeeded')]).get_output_in_json()

        self.cmd('az sf service show -g {rg} -c {cluster_name} --application-name {app_name} --service-name "{app_name}~testService"',
                 checks=[self.check('id', service['id'])])

        self.cmd('az sf application-type version create -g {rg} -c {cluster_name} '
                 '--application-type-name {app_type_name} --version {v2} --package-url {app_package_v2}',
                 checks=[self.check('provisioningState', 'Succeeded')])

        self.cmd('az sf application update -g {rg} -c {cluster_name} --application-name {app_name} --application-type-version {v2} '
                 '--application-parameters Precision=3 --health-check-stable-duration 0 --health-check-wait-duration 0 --health-check-retry-timeout 0 '
                 '--upgrade-domain-timeout 5000 --upgrade-timeout 7000 --failure-action Rollback --upgrade-replica-set-check-timeout 300 --force-restart',
                 checks=[self.check('provisioningState', 'Succeeded'),
                         self.check('typeVersion', '{v2}'),
                         self.check('parameters.Mode', 'binary'),
                         self.check('parameters.Precision', '3'),
                         self.check('upgradePolicy.forceRestart', True),
                         self.check('upgradePolicy.upgradeReplicaSetCheckTimeout', '00:05:00'),
                         self.check('upgradePolicy.rollingUpgradeMonitoringPolicy.healthCheckRetryTimeout', '00:00:00'),
                         self.check('upgradePolicy.rollingUpgradeMonitoringPolicy.healthCheckWaitDuration', '00:00:00'),
                         self.check('upgradePolicy.rollingUpgradeMonitoringPolicy.healthCheckStableDuration', '00:00:00'),
                         self.check('upgradePolicy.rollingUpgradeMonitoringPolicy.upgradeTimeout', '01:56:40'),
                         self.check('upgradePolicy.rollingUpgradeMonitoringPolicy.upgradeDomainTimeout', '01:23:20'),
                         self.check('upgradePolicy.rollingUpgradeMonitoringPolicy.failureAction', 'Rollback')])

        self.cmd('az sf application update -g {rg} -c {cluster_name} --application-name {app_name} --minimum-nodes 1 --maximum-nodes 3',
                 checks=[self.check('provisioningState', 'Succeeded')])
        self.cmd('az sf application show -g {rg} -c {cluster_name} --application-name {app_name}',
                 checks=[self.check('provisioningState', 'Succeeded'),
                         self.check('minimumNodes', 1),
                         self.check('maximumNodes', 3)])

        self.cmd('az sf application delete -g {rg} -c {cluster_name} --application-name {app_name}')
        self.cmd('az sf application-type delete -g {rg} -c {cluster_name} --application-type-name {app_type_name}')

        # SystemExit 3 'not found'
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('az sf application show -g {rg} -c {cluster_name} --application-name {app_name}')

    @ResourceGroupPreparer()
    def test_application(self):
        self.kwargs.update({
            'kv_name': self.create_random_name('sfrp-cli-kv-', 24),
            'loc': 'westus',
            'cert_name': self.create_random_name('sfrp-cli-', 24),
            'cluster_name': self.create_random_name('sfrp-cli-', 24),
            'vm_password': self.create_random_name('Pass@', 9),
            'app_type_name': 'CalcServiceApp',
            'v1': '1.0',
            'app_package_v1': 'https://sfrpserviceclienttesting.blob.core.windows.net/test-apps/CalcApp_1.0.sfpkg',
            'v2': '1.2',
            'app_package_v2': 'https://sfrpserviceclienttesting.blob.core.windows.net/test-apps/CalcApp_1.2.sfpkg',
            'app_name': self.create_random_name('testApp', 11),
            'service_type': 'CalcServiceType'
        })

        _create_cluster_with_separate_kv(self, self.kwargs)
        self._app_type_test()
        self._app_type_version_test()
        self._app_service_test()


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from __future__ import print_function

import time
import uuid
from contextlib import contextmanager

import os
from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer

try:
    import unittest.mock as mock
except ImportError:
    import mock

from azure.cli.testsdk import JMESPathCheck, JMESPathCheckExists, StringContainCheck
from azure_devtools.scenario_tests import AllowLargeResponse

NODE_STARTUP_TIME = 10 * 60  # Compute node should start in 10 mins after cluster creation.
CLUSTER_RESIZE_TIME = 20 * 60  # Cluster should resize in 20 mins after job submitted/completed.
LOCATION_FOR_SCENARIO_TESTS = 'eastus'
PASSWORD = str(uuid.uuid4())


def _data_file(filename):
    filepath = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', filename)
    return filepath.replace('\\', '\\\\')


class BatchAIEndToEndScenariosTest(ScenarioTest):
    @ResourceGroupPreparer(location=LOCATION_FOR_SCENARIO_TESTS)
    @StorageAccountPreparer(location=LOCATION_FOR_SCENARIO_TESTS)
    def test_batchai_manual_scale_scenario(self, resource_group, storage_account):
        # Typical usage scenario for regular (not auto scale) cluster.
        # 1. Create a compute cluster
        # 2. Execute some jobs on the cluster
        # 3. Resize the compute cluster to 0
        # 4. Resize the compute cluster to have some nodes
        # 5. Execute more jobs and examine execution results
        # 6. Delete the cluster
        # 7. Delete the jobs
        with self._given_configured_environment(resource_group, storage_account):
            # Create a file share 'share' to be mounted on the cluster
            self.cmd('az storage share create -n share')
            # Create a workspace
            self.cmd('az batchai workspace create -g {0} -n workspace'.format(resource_group),
                     checks=[JMESPathCheck('name', 'workspace')])
            self.cmd('az batchai workspace list -g {0}'.format(resource_group), checks=[JMESPathCheck("length(@)", 1)])
            # Create a cluster
            self.cmd('az batchai cluster create -g {0} -w workspace -n cluster -f "{1}"'.format(
                resource_group, _data_file('cluster_with_azure_files.json')),
                checks=[
                    JMESPathCheck('name', 'cluster'),
                    JMESPathCheck('allocationState', 'resizing'),
                    JMESPathCheck('scaleSettings.manual.targetNodeCount', 1),
                    JMESPathCheck('vmSize', 'STANDARD_D1'),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].accountName', storage_account),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].azureFileUrl',
                                  'https://{0}.file.core.windows.net/share'.format(storage_account)),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].credentialsInfo.accountKey', None),
                    JMESPathCheck('userAccountSettings.adminUserName', 'DemoUser'),
                    JMESPathCheck('userAccountSettings.adminUserPassword', None)])
            # Create an experiment
            self.cmd('az batchai experiment create -g {0} -w workspace -n experiment'.format(resource_group),
                     checks=[JMESPathCheck('name', 'experiment')])
            self.cmd('az batchai experiment list -g {0} -w workspace'.format(resource_group),
                     checks=[JMESPathCheck("length(@)", 1)])
            # Create the first job
            self.cmd('az batchai job create -c cluster -g {0} -w workspace -e experiment -n job -f "{1}"'.format(
                resource_group, _data_file('custom_toolkit_job.json')),
                checks=[
                    JMESPathCheck('name', 'job'),
                    JMESPathCheck('customToolkitSettings.commandLine',
                                  'echo hi | tee $AZ_BATCHAI_OUTPUT_OUTPUT/result.txt'),
                    JMESPathCheck('executionState', 'queued')])

            # Wait for the cluster to be allocated and job completed
            self.cmd('az batchai job wait -g {0} -w workspace -e experiment -n job'.format(resource_group))

            # Check the job's results
            self.cmd('az batchai job show -g {0} -w workspace -e experiment -n job'.format(resource_group), checks=[
                JMESPathCheck('name', 'job'),
                JMESPathCheck(
                    'customToolkitSettings.commandLine', 'echo hi | tee $AZ_BATCHAI_OUTPUT_OUTPUT/result.txt'),
                JMESPathCheck('executionState', 'succeeded'),
                JMESPathCheck('executionInfo.exitCode', 0),
                JMESPathCheck('executionInfo.errors', None),
            ])

            # Check the job's standard output: stdout.txt with length equal 3 ("hi\n"), stderr.txt
            self.cmd('az batchai job file list -g {0} -w workspace -e experiment -j job -d stdouterr'.format(
                resource_group), checks=[
                    JMESPathCheck("[].name | contains(@, 'execution.log')", True),
                    JMESPathCheck("[].name | contains(@, 'stderr.txt')", True),
                    JMESPathCheck("[].name | contains(@, 'stdout.txt')", True),
                    JMESPathCheck("[?name == 'stdout.txt'].contentLength", [3]),
                    JMESPathCheckExists("[0].downloadUrl"),
                    JMESPathCheckExists("[1].downloadUrl"),
            ])

            # Check the job's output directory
            self.cmd('az batchai job file list -g {0} -w workspace -e experiment -j job -d=OUTPUT'.format(
                resource_group),
                checks=[
                    JMESPathCheck("[].name | sort(@)", ['result.txt']),
                    JMESPathCheck("[0].contentLength", 3),  # hi/n
                    JMESPathCheckExists("[0].downloadUrl")
            ])

            # Resize the cluster to 0 nodes
            self.cmd('az batchai cluster resize -g {0} -w workspace -n cluster -t 0'.format(resource_group))
            time.sleep(NODE_STARTUP_TIME)

            # Cluster must be resized by this time
            self.cmd('az batchai cluster show -g {0} -w workspace -n cluster'.format(resource_group),
                     checks=[
                         JMESPathCheck('name', 'cluster'),
                         JMESPathCheck('currentNodeCount', 0),
                         JMESPathCheck('scaleSettings.manual.targetNodeCount', 0)
            ])

            # Resize the cluster to execute another job
            self.cmd('az batchai cluster resize -g {0} -w workspace -n cluster -t 1'.format(resource_group))

            # Create another job
            self.cmd('az batchai job create -c cluster -g {0} -w workspace -e experiment -n job2 -f "{1}"'.format(
                resource_group, _data_file('custom_toolkit_job.json')))

            # Wait for the cluster to finish resizing and job execution
            self.cmd('az batchai job wait -g {0} -w workspace -e experiment -n job2'.format(resource_group))

            # The job must succeed by this time
            self.cmd('az batchai job show -g {0} -w workspace -e experiment -n job2'.format(resource_group), checks=[
                JMESPathCheck('name', 'job2'),
                JMESPathCheck('executionState', 'succeeded'),
                JMESPathCheck('executionInfo.exitCode', 0),
                JMESPathCheck('executionInfo.errors', None),
            ])

            # Delete the cluster
            self.cmd('batchai cluster delete -g {0} -w workspace -n cluster -y'.format(resource_group))
            self.cmd('batchai cluster list -g {0} -w workspace'.format(resource_group),
                     checks=[JMESPathCheck("length(@)", 0)])

            # Delete the jobs
            self.cmd('az batchai job delete -g {0} -w workspace -e experiment -n job -y'.format(resource_group))
            self.cmd('az batchai job delete -g {0} -w workspace -e experiment -n job2 -y'.format(resource_group))
            self.cmd('az batchai job list -g {0} -w workspace -e experiment'.format(resource_group),
                     checks=[JMESPathCheck("length(@)", 0)])

    @ResourceGroupPreparer(location=LOCATION_FOR_SCENARIO_TESTS)
    @StorageAccountPreparer(location=LOCATION_FOR_SCENARIO_TESTS)
    def test_batchai_auto_scale_scenario(self, resource_group, storage_account):
        # Typical usage scenario for auto scale cluster.
        # 1. Create a compute cluster
        # 2. Submit a job
        # 3. The cluster will auto scale to execute the job
        # 4. Examine the job execution results
        # 5. The cluster will down scale
        with self._given_configured_environment(resource_group, storage_account):
            # Create a file share 'share' to be mounted on the cluster
            self.cmd('az storage share create -n share')
            # Create a workspace
            self.cmd('az batchai workspace create -g {0} -n workspace'.format(resource_group),
                     checks=[JMESPathCheck('name', 'workspace')])
            # Create a cluster
            self.cmd('az batchai cluster create -g {0} -w workspace -n cluster -f "{1}"'.format(
                resource_group, _data_file('auto_scale_cluster_with_azure_files.json')),
                checks=[
                    JMESPathCheck('name', 'cluster'),
                    JMESPathCheck('scaleSettings.autoScale.minimumNodeCount', 0),
                    JMESPathCheck('scaleSettings.autoScale.maximumNodeCount', 1),
                    JMESPathCheck('vmSize', 'STANDARD_D1'),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].accountName', storage_account),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].azureFileUrl',
                                  'https://{0}.file.core.windows.net/share'.format(storage_account)),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].credentialsInfo.accountKey', None),
                    JMESPathCheck('userAccountSettings.adminUserName', 'DemoUser'),
                    JMESPathCheck('userAccountSettings.adminUserPassword', None)])
            # Create an experiment
            self.cmd('az batchai experiment create -g {0} -w workspace -n experiment'.format(resource_group),
                     checks=[JMESPathCheck('name', 'experiment')])
            # Create the job
            self.cmd('az batchai job create -c cluster -g {0} -w workspace -e experiment -n job -f "{1}"'.format(
                resource_group, _data_file('custom_toolkit_job.json')),
                checks=[
                    JMESPathCheck('name', 'job'),
                    JMESPathCheck('customToolkitSettings.commandLine',
                                  'echo hi | tee $AZ_BATCHAI_OUTPUT_OUTPUT/result.txt'),
                    JMESPathCheck('executionState', 'queued')])

            # Wait for the cluster to scale up and job completed
            self.cmd('az batchai job wait -g {0} -w workspace -e experiment -n job'.format(resource_group))

            # The job must succeed by this time
            self.cmd('az batchai job show -g {0} -w workspace -e experiment -n job'.format(resource_group), checks=[
                JMESPathCheck('name', 'job'),
                JMESPathCheck(
                    'customToolkitSettings.commandLine', 'echo hi | tee $AZ_BATCHAI_OUTPUT_OUTPUT/result.txt'),
                JMESPathCheck('executionState', 'succeeded'),
                JMESPathCheck('executionInfo.exitCode', 0),
                JMESPathCheck('executionInfo.errors', None),
            ])

            # Give cluster a time do down scale
            time.sleep(CLUSTER_RESIZE_TIME)

            # By this time the cluster should not have any nodes
            self.cmd('az batchai cluster show -g {0} -w workspace -n cluster'.format(resource_group),
                     checks=JMESPathCheck('currentNodeCount', 0))

    @ResourceGroupPreparer(location=LOCATION_FOR_SCENARIO_TESTS)
    @StorageAccountPreparer(location=LOCATION_FOR_SCENARIO_TESTS)
    def test_batchai_cluster_with_file_systems(self, resource_group, storage_account):
        # Tests creation of a cluster with mounted file systems defined in config.
        # 1. Create an Azure File Share and Azure Blob Container to mount on the cluster.
        # 2. Create a cluster and verify parameters.
        # 3. Verify that cluster was able to start nodes.
        with self._given_configured_environment(resource_group, storage_account):
            # Create a file share 'share' and blob container 'container' to be mounted on cluster nodes.
            self.cmd('az storage share create -n share')
            self.cmd('az storage container create -n container')
            self.cmd('az batchai workspace create -g {0} -n workspace'.format(resource_group),
                     checks=[JMESPathCheck('name', 'workspace')])
            self.cmd(
                'az batchai cluster create -g {0} -w workspace -n cluster -f "{1}" '
                '--afs-name share --bfs-name container '
                '-u DemoUser -k "{2}"'.format(resource_group, _data_file('simple_cluster.json'), _data_file('key.txt')),
                checks=[
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].accountName', storage_account),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].azureFileUrl',
                                  'https://{0}.file.core.windows.net/share'.format(storage_account)),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].relativeMountPath', 'afs'),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].credentialsInfo.accountKey', None),
                    JMESPathCheck('nodeSetup.mountVolumes.azureBlobFileSystems[0].accountName', storage_account),
                    JMESPathCheck('nodeSetup.mountVolumes.azureBlobFileSystems[0].containerName', 'container'),
                    JMESPathCheck('nodeSetup.mountVolumes.azureBlobFileSystems[0].relativeMountPath', 'bfs'),
                    JMESPathCheck('nodeSetup.mountVolumes.azureBlobFileSystems[0].credentialsInfo.accountKey', None),
                    JMESPathCheck('userAccountSettings.adminUserName', 'DemoUser'),
                    JMESPathCheck('userAccountSettings.adminUserPassword', None)])

            # Give file server and cluster to finish preparation.
            time.sleep(NODE_STARTUP_TIME * 2)

            # Check the node in the cluster successfully started - was able to mount nfs and azure filesystem.
            self.cmd('az batchai cluster show -g {0} -w workspace -n cluster'.format(resource_group),
                     checks=[JMESPathCheck('nodeStateCounts.idleNodeCount', 1)])

    @ResourceGroupPreparer(location=LOCATION_FOR_SCENARIO_TESTS)
    @StorageAccountPreparer(location=LOCATION_FOR_SCENARIO_TESTS)
    def test_batchai_config_less_cluster_with_file_systems(self, resource_group, storage_account):
        # Test creation of a cluster with mount file systems defined via command line.
        with self._given_configured_environment(resource_group, storage_account):
            self.cmd('az storage share create -n share')
            self.cmd('az storage container create -n container')
            self.cmd('az batchai workspace create -g {0} -n workspace'.format(resource_group))
            self.cmd(
                'az batchai cluster create -g {0} -w workspace -n cluster '
                '-i UbuntuLTS --vm-size STANDARD_D1 --min 1 --max 1 -u DemoUser -k "{1}" '
                '--afs-name share --bfs-name container'.format(
                    resource_group, _data_file('key.txt')),
                checks=[
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].accountName', storage_account),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].azureFileUrl',
                                  'https://{0}.file.core.windows.net/share'.format(storage_account)),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].relativeMountPath', 'afs'),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].credentialsInfo.accountKey', None),
                    JMESPathCheck('nodeSetup.mountVolumes.azureBlobFileSystems[0].accountName', storage_account),
                    JMESPathCheck('nodeSetup.mountVolumes.azureBlobFileSystems[0].containerName', 'container'),
                    JMESPathCheck('nodeSetup.mountVolumes.azureBlobFileSystems[0].relativeMountPath', 'bfs'),
                    JMESPathCheck('nodeSetup.mountVolumes.azureBlobFileSystems[0].credentialsInfo.accountKey', None),
                    JMESPathCheck('userAccountSettings.adminUserName', 'DemoUser'),
                    JMESPathCheck('userAccountSettings.adminUserPassword', None)])

            # Give file server and cluster to finish preparation.
            time.sleep(NODE_STARTUP_TIME * 2)

            # Check the node in the cluster successfully started - was able to mount nfs and azure filesystem.
            self.cmd('az batchai cluster show -g {0} -w workspace -n cluster'.format(resource_group),
                     checks=[JMESPathCheck('nodeStateCounts.idleNodeCount', 1)])

    @AllowLargeResponse()
    @ResourceGroupPreparer(location=LOCATION_FOR_SCENARIO_TESTS)
    @StorageAccountPreparer(name_prefix='bai', location=LOCATION_FOR_SCENARIO_TESTS)
    def test_batchai_cluster_with_auto_storage(self, resource_group, storage_account):
        # Test creation of a cluster with auto-storage account.
        with mock.patch('azure.cli.command_modules.batchai.custom._get_auto_storage_resource_group') as p:
            p.return_value = resource_group
            self.cmd('az batchai workspace create -g {0} -n workspace'.format(resource_group))
            self.cmd(
                'az batchai cluster create -g {0} -w workspace -n cluster '
                '-i UbuntuLTS --vm-size STANDARD_D1 -t 0 -u DemoUser -k "{1}" '
                '--use-auto-storage'.format(
                    resource_group, _data_file('key.txt')),
                checks=[
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].accountName', storage_account),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].azureFileUrl',
                                  'https://{0}.file.core.windows.net/batchaishare'.format(storage_account)),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].relativeMountPath', 'autoafs'),
                    JMESPathCheck('nodeSetup.mountVolumes.azureBlobFileSystems[0].accountName', storage_account),
                    JMESPathCheck('nodeSetup.mountVolumes.azureBlobFileSystems[0].containerName', 'batchaicontainer'),
                    JMESPathCheck('nodeSetup.mountVolumes.azureBlobFileSystems[0].relativeMountPath', 'autobfs')])

    @AllowLargeResponse()
    @ResourceGroupPreparer(location=LOCATION_FOR_SCENARIO_TESTS)
    @StorageAccountPreparer(name_prefix='bai', location=LOCATION_FOR_SCENARIO_TESTS)
    def test_batchai_cluster_with_setup_command(self, resource_group, storage_account):
        # Test creation of a cluster with auto-storage account and setup task.
        with mock.patch('azure.cli.command_modules.batchai.custom._get_auto_storage_resource_group') as p:
            p.return_value = resource_group
            self.cmd('az batchai workspace create -g {0} -n workspace'.format(resource_group))
            self.cmd(
                'az batchai cluster create -n cluster -g {0} -w workspace -s STANDARD_D1 -t 0 -u DemoUser -k "{1}" '
                '--use-auto-storage --setup-task "echo hi" --setup-task-output "$AZ_BATCHAI_MOUNT_ROOT/autoafs"'.format(
                    resource_group, _data_file('key.txt')),
                checks=[
                    JMESPathCheck('nodeSetup.setupTask.commandLine', 'echo hi'),
                    JMESPathCheck('nodeSetup.setupTask.stdOutErrPathPrefix', '$AZ_BATCHAI_MOUNT_ROOT/autoafs'),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].accountName', storage_account),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].azureFileUrl',
                                  'https://{0}.file.core.windows.net/batchaishare'.format(storage_account)),
                    JMESPathCheck('nodeSetup.mountVolumes.azureFileShares[0].relativeMountPath', 'autoafs'),
                    JMESPathCheck('nodeSetup.mountVolumes.azureBlobFileSystems[0].accountName', storage_account),
                    JMESPathCheck('nodeSetup.mountVolumes.azureBlobFileSystems[0].containerName', 'batchaicontainer'),
                    JMESPathCheck('nodeSetup.mountVolumes.azureBlobFileSystems[0].relativeMountPath', 'autobfs')])

    @ResourceGroupPreparer(location=LOCATION_FOR_SCENARIO_TESTS)
    @StorageAccountPreparer(location=LOCATION_FOR_SCENARIO_TESTS)
    def test_batchai_job_level_mounting_scenario(self, resource_group, storage_account):
        # Typical usage scenario for regular (not auto scale) cluster.
        # 1. Create a compute cluster.
        # 2. Execute a job with job level filesystems when file systems specified in config file.
        # 3. Check the job succeeded and files are generated.
        # 4. Execute a job with job level filesystems when file systems specified via command line.
        # 5. Check the job succeeded and files are generated.
        with self._given_configured_environment(resource_group, storage_account):
            # Create a file share 'share' to be mounted on the cluster
            self.cmd('az storage share create -n share')
            self.cmd('az storage container create -n container')
            # Create a workspace
            self.cmd('az batchai workspace create -g {0} -n workspace'.format(resource_group))
            # Create a cluster
            self.cmd('az batchai cluster create -g {0} -w workspace -n cluster -f "{1}" -u DemoUser -k "{2}"'.format(
                resource_group, _data_file('simple_cluster.json'), _data_file('key.txt')))
            # Create an experiment
            self.cmd('az batchai experiment create -g {0} -w workspace -n experiment'.format(resource_group))
            # Submit the job which has mount_volumes in the config file.
            self.cmd('az batchai job create -c cluster -g {0} -w workspace -e experiment -n job -f "{1}"'.format(
                resource_group, _data_file('job_with_file_systems.json')))
            # Wait for the cluster to be allocated and job completed
            self.cmd('az batchai job wait -g {0} -w workspace -e experiment -n job'.format(resource_group))
            # Check the job's results
            self.cmd('az batchai job show -g {0} -w workspace -e experiment -n job'.format(resource_group), checks=[
                JMESPathCheck('name', 'job'),
                JMESPathCheck(
                    'customToolkitSettings.commandLine', 'echo hi | tee $AZ_BATCHAI_OUTPUT_OUTPUT/result.txt'),
                JMESPathCheck('executionState', 'succeeded'),
                JMESPathCheck('executionInfo.exitCode', 0),
                JMESPathCheck('executionInfo.errors', None),
            ])
            # Check the job's standard output: stdout.txt with length equal 3 ("hi\n"), stderr.txt
            self.cmd('az batchai job file list -g {0} -w workspace -e experiment -j job -d stdouterr'.format(
                resource_group), checks=[
                    JMESPathCheck("[].name | contains(@, 'execution.log')", True),
                    JMESPathCheck("[].name | contains(@, 'stderr.txt')", True),
                    JMESPathCheck("[].name | contains(@, 'stdout.txt')", True),
                    JMESPathCheck("[?name == 'stdout.txt'].contentLength", [3]),
                    JMESPathCheckExists("[0].downloadUrl"),
                    JMESPathCheckExists("[1].downloadUrl"),
            ])
            # Check the job's output directory
            self.cmd('az batchai job file list -g {0} -w workspace -e experiment -j job -d OUTPUT'.format(
                resource_group), checks=[
                JMESPathCheck("[].name | contains(@, 'result.txt')", True),
                JMESPathCheck("[0].contentLength", 3),  # hi/n
                JMESPathCheckExists("[0].downloadUrl")
            ])
            # Submit the job specifying Azure File Share and Azure Blob Container via command line args
            self.cmd('az batchai job create -c cluster -g {0} -w workspace -e experiment -n job2 -f "{1}" '
                     '--afs-name share --bfs-name container'.format(resource_group,
                                                                    _data_file('job_referencing_file_systems.json')))
            # Wait for the cluster to be allocated and job completed
            self.cmd('batchai job wait -g {0} -w workspace -e experiment -n job2'.format(resource_group))
            # Check the job's results
            self.cmd('batchai job show -g {0} -w workspace -e experiment -n job2'.format(resource_group), checks=[
                JMESPathCheck('name', 'job2'),
                JMESPathCheck(
                    'customToolkitSettings.commandLine', 'echo hi | tee $AZ_BATCHAI_OUTPUT_OUTPUT/result.txt'),
                JMESPathCheck('executionState', 'succeeded'),
                JMESPathCheck('executionInfo.exitCode', 0),
                JMESPathCheck('executionInfo.errors', None),
            ])
            # Check the job's standard output: stdout.txt with length equal 3 ("hi\n"), stderr.txt
            self.cmd('batchai job file list -g {0} -w workspace -e experiment -j job2 -d stdouterr'.format(
                resource_group), checks=[
                JMESPathCheck("[].name | contains(@, 'execution.log')", True),
                JMESPathCheck("[].name | contains(@, 'stderr.txt')", True),
                JMESPathCheck("[].name | contains(@, 'stdout.txt')", True),
                JMESPathCheck("[?name == 'stdout.txt'].contentLength", [3]),
                JMESPathCheckExists("[0].downloadUrl"),
                JMESPathCheckExists("[1].downloadUrl"),
            ])
            # Check the job's output directory
            self.cmd('batchai job file list -g {0} -w workspace -e experiment -j job2 -d OUTPUT'.format(
                resource_group), checks=[
                JMESPathCheck("[].name | contains(@, 'result.txt')", True),
                JMESPathCheck("[0].contentLength", 3),  # hi/n
                JMESPathCheckExists("[0].downloadUrl")
            ])

    def test_batchai_usages(self):
        # Just check if we can get a usage and it contains info about clusters.
        self.cmd('batchai list-usages -l {0}'.format(LOCATION_FOR_SCENARIO_TESTS), checks=[
            StringContainCheck("Cluster")])
        self.cmd('batchai list-usages -l {0} -o table'.format(LOCATION_FOR_SCENARIO_TESTS), checks=[
            StringContainCheck("Cluster")])

    @contextmanager
    def _given_configured_environment(self, resource_group, storage_account):

        # Configure storage account related environment variables.
        account_key = self.cmd('storage account keys list -n {} -g {} --query "[0].value" -otsv'.format(
            storage_account, resource_group)).output[:-1]
        self.set_env('AZURE_STORAGE_ACCOUNT', storage_account)
        self.set_env('AZURE_STORAGE_KEY', account_key)
        self.set_env('AZURE_BATCHAI_STORAGE_ACCOUNT', storage_account)
        self.set_env('AZURE_BATCHAI_STORAGE_KEY', account_key)
        try:
            yield
        finally:
            self.pop_env('AZURE_STORAGE_ACCOUNT')
            self.pop_env('AZURE_STORAGE_KEY')
            self.pop_env('AZURE_BATCHAI_STORAGE_ACCOUNT')
            self.pop_env('AZURE_BATCHAI_STORAGE_KEY')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure_devtools.scenario_tests import mock_in_unit_test
from azure_devtools.scenario_tests.const import MOCKED_SUBSCRIPTION_ID, MOCKED_TENANT_ID

from .exceptions import CliExecutionError

MOCKED_USER_NAME = 'example@example.com'


def patch_progress_controller(unit_test):
    def _mock_pass(*args, **kwargs):  # pylint: disable=unused-argument
        pass

    mock_in_unit_test(
        unit_test, 'azure.cli.core.commands.progress.ProgressHook.update', _mock_pass)
    mock_in_unit_test(
        unit_test, 'azure.cli.core.commands.progress.ProgressHook.add', _mock_pass)
    mock_in_unit_test(
        unit_test, 'azure.cli.core.commands.progress.ProgressHook.end', _mock_pass)


def patch_main_exception_handler(unit_test):
    from vcr.errors import CannotOverwriteExistingCassetteException

    def _handle_main_exception(ex, *args, **kwargs):  # pylint: disable=unused-argument
        if isinstance(ex, CannotOverwriteExistingCassetteException):
            # This exception usually caused by a no match HTTP request. This is a product error
            # that is caused by change of SDK invocation.
            raise ex

        raise CliExecutionError(ex)

    mock_in_unit_test(unit_test, 'azure.cli.core.util.handle_exception', _handle_main_exception)


def patch_load_cached_subscriptions(unit_test):
    def _handle_load_cached_subscription(*args, **kwargs):  # pylint: disable=unused-argument

        return [{
            "id": MOCKED_SUBSCRIPTION_ID,
            "user": {
                "name": MOCKED_USER_NAME,
                "type": "user"
            },
            "state": "Enabled",
            "name": "Example",
            "tenantId": MOCKED_TENANT_ID,
            "isDefault": True}]

    mock_in_unit_test(unit_test,
                      'azure.cli.core._profile.Profile.load_cached_subscriptions',
                      _handle_load_cached_subscription)


def patch_retrieve_token_for_user(unit_test):
    def _retrieve_token_for_user(*args, **kwargs):  # pylint: disable=unused-argument
        import datetime
        fake_token = 'top-secret-token-for-you'
        return 'Bearer', fake_token, {
            "tokenType": "Bearer",
            "expiresIn": 3600,
            "expiresOn": (datetime.datetime.now() + datetime.timedelta(hours=1)).strftime("%Y-%m-%d %H:%M:%S.%f"),
            "resource": args[3],
            "accessToken": fake_token,
            "refreshToken": fake_token
        }

    mock_in_unit_test(unit_test,
                      'azure.cli.core._profile.CredsCache.retrieve_token_for_user',
                      _retrieve_token_for_user)


def patch_long_run_operation_delay(unit_test):
    def _shortcut_long_run_operation(*args, **kwargs):  # pylint: disable=unused-argument
        return

    mock_in_unit_test(unit_test,
                      'msrestazure.azure_operation.AzureOperationPoller._delay',
                      _shortcut_long_run_operation)
    mock_in_unit_test(unit_test,
                      'msrestazure.polling.arm_polling.ARMPolling._delay',
                      _shortcut_long_run_operation)
    mock_in_unit_test(unit_test,
                      'azure.cli.core.commands.LongRunningOperation._delay',
                      _shortcut_long_run_operation)


def patch_get_current_system_username(unit_test):
    def _get_current_system_username(*args, **kwargs):  # pylint: disable=unused-argument
        from .utilities import create_random_name
        return create_random_name(prefix='example_')

    mock_in_unit_test(unit_test,
                      'azure.cli.core.local_context._get_current_system_username',
                      _get_current_system_username)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from datetime import datetime, timedelta

import uuid
import re
import json
import isodate

from knack.log import get_logger
logger = get_logger(__name__)

iso8601pattern = re.compile("^P(?!$)(\\d+Y)?(\\d+M)?(\\d+W)?(\\d+D)?(T(?=\\d)(\\d+H)?(\\d+M)?(\\d+.)?(\\d+S)?)?$")


def _gen_guid():
    return uuid.uuid4()


def _is_guid(guid):
    try:
        uuid.UUID(guid)
        return True
    except ValueError:
        pass
    return False


def parse_duration(str_duration):
    if str_duration:
        if iso8601pattern.match(str_duration):
            return isodate.parse_duration(str_duration)

        return parse_timedelta(str_duration)
    return None


def parse_timedelta(str_duration):
    if str_duration:
        datetime_duration = datetime.strptime(str_duration, '%H:%M:%S')
    return timedelta(hours=datetime_duration.hour or 0,
                     minutes=datetime_duration.minute or 0,
                     seconds=datetime_duration.second or 0)


def camel_to_snake(name):
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()


def snake_to_camel_case(snake_str):
    components = snake_str.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])


class JsonBytearrayEncoder(json.JSONEncoder):
    DATE_FORMAT = "%Y-%m-%d"
    TIME_FORMAT = "%H:%M:%S"

    def default(self, obj):  # pylint: disable=E0202,W0221
        if isinstance(obj, datetime):
            return obj.strftime("%s %s" % (self.DATE_FORMAT, self.TIME_FORMAT))

        if isinstance(obj, bytearray):
            return bytes(obj).decode('utf-8', 'ignore')

        try:
            return obj.toJSON()
        except Exception:  # pylint: disable=W0703
            obj = vars(obj)
            obj.pop('additional_properties', None)
            keys = list(obj.keys())
            for key in keys:
                obj[snake_to_camel_case(key)] = obj.pop(key)
            return obj


def create_ip_range(resource_name, ip):
    from azure.mgmt.media.models import IPRange
    subnet_prefix_length = None
    try:
        if '/' in ip:
            ipSplit = ip.split('/')
            ip = ipSplit[0]
            subnet_prefix_length = ipSplit[1]
    except ValueError:
        pass
    return IPRange(name=("{}_{}".format(resource_name, ip)), address=ip, subnet_prefix_length=subnet_prefix_length)


# The Media Services SDK does not throw an exception for 404s.
# This helper will make our 'not founds' look like other services.
def show_resource_not_found_message(group, account, ams_type, name):
    import sys
    message = build_resource_not_found_message(group, account, ams_type, name)
    from azure.cli.core.azlogging import CommandLoggerContext
    with CommandLoggerContext(logger):
        logger.error(message)
        sys.exit(3)


def build_resource_not_found_message(group, account, ams_type, name):
    message = "The Resource 'Microsoft.Media/mediaServices/{}/{}/{}' under resource group '{}' was not found." \
        .format(account, ams_type, name, group)
    return message


def show_child_resource_not_found_message(group, account, ams_type, name, ams_child_type, child_name):
    import sys
    message = build_child_resource_not_found_message(account, ams_type, name, ams_child_type, child_name, group)
    from azure.cli.core.azlogging import CommandLoggerContext
    with CommandLoggerContext(logger):
        logger.error(message)
        sys.exit(3)


def build_child_resource_not_found_message(group, account, ams_type, name, ams_child_type, child_name):
    message = "The Resource 'Microsoft.Media/mediaServices/{}/{}/{}/{}/{}' under resource group '{}' was not found." \
        .format(account, ams_type, name, ams_child_type, child_name, group)
    return message



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os.path
import unittest
import mock

from azure.cli.core.cloud import CloudEndpointNotSetException
from azure.cli.core.mock import DummyCli

from knack.util import CLIError


def _get_test_cmd():
    from azure.cli.core.mock import DummyCli
    from azure.cli.core import AzCommandsLoader
    from azure.cli.core.commands import AzCliCommand
    from azure.cli.core.profiles import ResourceType
    cli_ctx = DummyCli()
    loader = AzCommandsLoader(cli_ctx, resource_type=ResourceType.MGMT_COMPUTE)
    cmd = AzCliCommand(loader, 'test', None)
    cmd.command_kwargs = {'resource_type': ResourceType.MGMT_COMPUTE}
    cmd.cli_ctx = cli_ctx
    return cmd


class TestVMImage(unittest.TestCase):
    @mock.patch('azure.cli.command_modules.vm.custom.urlopen', autospec=True)
    def test_read_images_from_alias_doc(self, mock_urlopen):
        from azure.cli.command_modules.vm.custom import list_vm_images
        cmd = _get_test_cmd()
        file_path = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                 'aliases.json')
        with open(file_path, 'r') as test_file:
            test_data = test_file.read().encode()

        mock_read = mock.MagicMock()
        mock_read.read.return_value = test_data
        mock_urlopen.return_value = mock_read

        # action
        images = list_vm_images(cmd)

        # assert
        win_images = [i for i in images if i['publisher'] == 'MicrosoftWindowsServer']
        self.assertTrue(len(win_images) > 0)
        ubuntu_image = next(i for i in images if i['publisher'] == 'Canonical')
        self.assertEqual(ubuntu_image['publisher'], 'Canonical')
        self.assertEqual(ubuntu_image['offer'], 'UbuntuServer')
        self.assertEqual(ubuntu_image['urnAlias'], 'UbuntuLTS')
        parts = ubuntu_image['urn'].split(':')
        self.assertEqual(parts[0], ubuntu_image['publisher'])
        self.assertEqual(parts[1], ubuntu_image['offer'])
        self.assertEqual(parts[2], ubuntu_image['sku'])
        self.assertEqual(parts[3], ubuntu_image['version'])

    @mock.patch('azure.cli.core.cloud.get_active_cloud', autospec=True)
    def test_when_alias_doc_is_missing(self, mock_get_active_cloud):
        from azure.cli.command_modules.vm._actions import load_images_from_aliases_doc
        p = mock.PropertyMock(side_effect=CloudEndpointNotSetException(''))
        mock_cloud = mock.MagicMock()
        type(mock_cloud.endpoints).vm_image_alias_doc = p
        mock_get_active_cloud.return_value = mock_cloud
        # assert
        cli_ctx = DummyCli()
        cli_ctx.cloud = mock_cloud
        images = load_images_from_aliases_doc(cli_ctx)
        self.assertEqual(images[0], {'urnAlias': 'CentOS', 'publisher': 'OpenLogic',
                                     'offer': 'CentOS', 'sku': '7.5', 'version': 'latest'})


if __name__ == '__main__':
    unittest.main()
